// ----------------------------------------------------------------------------
//
// Snoop Component Suite version 3.0
//
// http://www.netlab.co.kr
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------

unit Snoop;

interface

uses
  Windows, Classes, SysUtils, WinSock, IpHlpApi,
  Bpf, IpTypes, Pcap, VDThread, VDTrace;

// ----------------------------------------------------------------------------
// Constant
// ----------------------------------------------------------------------------
const
  SNOOP_DEFAULT_READTIMEOUT     = 1;
  SNOOP_DEFAULT_SNAPLEN         = 65535;
  SNOOP_MAC_SIZE                = 6;
  SNOOP_TIMEOUT                 = 5000;

// ----------------------------------------------------------------------------
// Type Definition
// ----------------------------------------------------------------------------
type
  TPcap                      = pcap_t;
  PPcap                      = ppcap_t;
  PCAP_PKTHDR                = Pcap.pcap_pkthdr;
  PPCAP_PKTHDR               = ^PCAP_PKTHDR;
  PPPCAP_PKTHDR              = ^PPCAP_PKTHDR;

  PPChar                     = ^PChar;
  UINT8                      = BYTE;
  PUINT8                     = ^UINT8;
  UINT16                     = WORD;
  PUINT16                    = ^UINT16;
  UINT32                     = DWORD;
  PUINT32                    = ^UINT32;
  MACADDR                    = array [0 .. SNOOP_MAC_SIZE - 1] of UINT8;
  PMACADDR                   = ^MACADDR;
  UINT                       = LongWord;

// ----------------------------------------------------------------------------
// Data-link level type codes
// ----------------------------------------------------------------------------
const
  DLT_NULL                   = 0;  // BSD loopback encapsulation
  DLT_EN10MB                 = 1;  // Ethernet (10Mb)
  DLT_EN3MB                  = 2;  // Experimental Ethernet (3Mb)
  DLT_AX25                   = 3;  // Amateur Radio AX.25
  DLT_PRONET                 = 4;  // Proteon ProNET Token Ring
  DLT_CHAOS                  = 5;  // Chaos
  DLT_IEEE802                = 6;  // IEEE 802 Networks
  DLT_ARCNET                 = 7;  // ARCNET, with BSD-style header
  DLT_SLIP                   = 8;  // Serial Line IP
  DLT_PPP                    = 9;  // Point-to-point Protocol
  DLT_FDDI                   = 10; // FDDI

// ----------------------------------------------------------------------------
// ETH Hdr
// ----------------------------------------------------------------------------
const
  PROTO_IP                   = $0800;
  PROTO_ARP                  = $0806;
  PROTO_XNS                  = $0600;
  PROTO_SNMP                 = $814C;
  PROTO_OLD_IPX              = $8137;
  PROTO_NOVELL               = $8138;
  PROTO_IPNG                 = $86DD;

 type
  ETH_HDR = packed record
    Destination:             MACADDR;
    Source:                  MACADDR;
    Protocol:                UINT16;
  end;
  PETH_HDR = ^ETH_HDR;

// ----------------------------------------------------------------------------
// IP Hdr(IPv4)
// ----------------------------------------------------------------------------
const
  PROTO_ICMP                 = $01;
  PROTO_TCP                  = $06;
  PROTO_UDP                  = $11;

type
  IP_HDR = packed record
    VerLen:                  UINT8;
    Service:                 UINT8;
    Len:                     UINT16;
    Ident:                   UINT16;
    FlagOff:                 UINT16;
    TimeLive:                UINT8;
    Protocol:                UINT8;
    Checksum:                UINT16;
    Source:                  UINT32;
    Destination:             UINT32;
  end;
  PIP_HDR = ^IP_HDR;
  PPIP_HDR = ^PIP_HDR;

// ----------------------------------------------------------------------------
// ARP Hdr
// ----------------------------------------------------------------------------
const
  ARP_REQUEST                = $0001;
  ARP_REPLY                  = $0002;

type
  ARP_HDR = packed record
    HardwareType:            UINT16;
    ProtocolType:            UINT16;
    HLen:                    UINT8;
    PLen:                    UINT8;
    Operation:               UINT16;
    SenderHA:                MACADDR;
    SenderIP:                UINT32;
    TargetHA:                MACADDR;
    TargetIP:                UINT32;
  end;
  PARP_HDR = ^ARP_HDR;
  PPARP_HDR = ^PARP_HDR;

// ----------------------------------------------------------------------------
// ICMP Hdr
// ----------------------------------------------------------------------------
type
  ICMP_HDR = packed record
    Type_:                   UINT8;
    Code:                    UINT8;
    Checksum:                UINT16;
    Data:                    PChar;
  end;
  PICMP_HDR = ^ICMP_HDR;
  PPICMP_HDR = ^PICMP_HDR;

// ----------------------------------------------------------------------------
// TCP Hdr
// ----------------------------------------------------------------------------
const
  TCP_FLAG_FIN               = $01;
  TCP_FLAG_SYN               = $02;
  TCP_FLAG_RST               = $04;
  TCP_FLAG_PSH               = $08;
  TCP_FLAG_ACK               = $10;
  TCP_FLAG_URG               = $20;

type
  TCP_HDR = packed record
    Source:                  UINT16;
    Destination:             UINT16;
    Seq:                     UINT32;
    Ack:                     UINT32;
    Off_Rsvd:                UINT8;
    Rsvd_Flags:              UINT8;
    Window:                  UINT16;
    Checksum:                UINT16;
    UrgPoint:                UINT16;
  end;
  PTCP_HDR = ^TCP_HDR;
  PPTCP_HDR = ^PTCP_HDR;

// ----------------------------------------------------------------------------
// UDP Hdr
// ----------------------------------------------------------------------------
type
  UDP_HDR = packed record
    Source:                  UINT16;
    Destination:             UINT16;
    Len:                     UINT16;
    Checksum:                UINT16;
  end;
  PUDP_HDR = ^UDP_HDR;
  PPUDP_HDR = ^PUDP_HDR;

// ----------------------------------------------------------------------------
// Event Definition
// ----------------------------------------------------------------------------
  TOnGetRemoteAdapterInfo = procedure (
    Sender: TObject;
    AdapterNames: TStringList;
    AdapterDescriptions: TStringList;
    var AdapterIndex: Integer) of object;
  TOnCaptureEvent = procedure (
    Sender: TObject;
    PKTHdr: PPCAP_PKTHDR;
    ETHHdr: PETH_HDR) of object;
  TOnCaptureIPEvent = procedure (
    Sender: TObject;
    PKTHdr: PPCAP_PKTHDR;
    ETHHdr: PETH_HDR;
    IPHdr:  PIP_HDR) of object;
  TOnCaptureARPEvent = procedure (
    Sender: TObject;
    PKTHdr: PPCAP_PKTHDR;
    ETHHdr: PETH_HDR;
    ARPHdr: PARP_HDR) of object;
  TOnCaptureICMPEvent = procedure (
    Sender: TObject;
    PKTHdr: PPCAP_PKTHDR;
    ETHHdr: PETH_HDR;
    IPHdr:  PIP_HDR;
    ICMPHdr: PICMP_HDR) of object;
  TOnCaptureTCPEvent = procedure (
    Sender: TObject;
    PKTHdr: PPCAP_PKTHDR;
    ETHHdr: PETH_HDR;
    IPHdr:  PIP_HDR;
    TCPHdr: PTCP_HDR) of object;
  TOnCaptureUDPEvent = procedure (
    Sender: TObject;
    PKTHdr: PPCAP_PKTHDR;
    ETHHdr: PETH_HDR;
    IPHdr:  PIP_HDR;
    UDPHdr: PUDP_HDR) of object;
  TOnCaptureTCPDataEvent = procedure (
    Sender: TObject;
    PKTHdr: PPCAP_PKTHDR;
    ETHHdr: PETH_HDR;
    IPHdr:  PIP_HDR;
    TCPHdr: PTCP_HDR;
    TCPData: PChar;
    TCPDataLen: Integer) of object;
  TOnCaptureUDPDataEvent = procedure (
    Sender: TObject;
    PKTHdr: PPCAP_PKTHDR;
    ETHHdr: PETH_HDR;
    IPHdr:  PIP_HDR;
    UDPHdr: PUDP_HDR;
    UDPData: PChar;
    UDPDataLen: Integer) of object;
  TOnCaptureStatisticsEvent = procedure (
    Sender: TObject;
    PKTHdr: PPCAP_PKTHDR;
    Packets: Int64;
    Bytes: Int64) of object;

// ----------------------------------------------------------------------------
// TSnoopThread
// ----------------------------------------------------------------------------
  TCustomSnoop = class;
  TSnoopThread = class(TVDThread)
  protected
    FSnoop: TCustomSnoop;
  public
    property Snoop: TCustomSnoop read FSnoop write FSnoop;
  public
    constructor Create(Snoop: TCustomSnoop);
    destructor Destroy; override;
    function Open: Boolean;
    function Close: Boolean;
    procedure Run; override; // gilgil ltemp 2006.10.24
    // procedure Execute; override;
  end;

// ----------------------------------------------------------------------------
// TCustomSnoop
// ----------------------------------------------------------------------------
  TCustomSnoop = class(TComponent)
  protected
    // property
    FActive: Boolean;
    FAdapterIndex: Integer;
    FAdapterName: String;
    FAdapterDescription: String;
    FAdapterNames: TStringList;
    FAdapterDescriptions: TStringList;

    FFilter: String;
    FFlags: Integer;
    FReadTimeout: Integer;
    FSnapLen: Integer;
    FThreadSafe: Boolean;

    FCaptureEnabled: Boolean;
    FError: String;
    FSourceName: String;
    FOnGetRemoteAdapterInfo: TOnGetRemoteAdapterInfo;
    procedure SetAdapterIndex(Value: Integer);
    procedure SetAdapterName(Value: String);
    procedure SetAdapterDescription(Value: String);
    function DoOpen(Source: PChar; Auth: ppcap_rmtauth; Dev: ppcap_if_t): Boolean;
    procedure CreateSnoopThread(AutoResume: Boolean = true);
  protected
    property OnGetRemoteAdapterInfo: TOnGetRemoteAdapterInfo
      read FOnGetRemoteAdapterInfo write FOnGetRemoteAdapterInfo;
  protected
    // used internally
    FThread: TSnoopThread;
    FPcap: PPcap;
    FPKTHdr: PPCAP_PKTHDR;
    FETHHdr: PETH_HDR;
    procedure OnCaptureCB; virtual; abstract;
    procedure GetAdapterNames;
    procedure GetAdapterDescriptions;
    function ProcessFilter(Dev: ppcap_if_t): Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Open(AutoResume: Boolean = true): Boolean; overload;
    function Open(Host: String; UserName: String; Password: String): Boolean; overload;
    function LoadFromFile(FileName: String): Boolean; overload;
    function Close: Boolean;
  public
    property Active: Boolean read FActive;
    property Thread: TSnoopThread read FThread write FThread;
    property Pcap: PPcap read FPcap write FPcap;
    property AdapterNames: TStringList read FAdapterNames;
    property AdapterDescriptions: TStringList read FAdapterDescriptions;
    property Error: String read FError write FError;
    property SourceName: String read FSourceName;
  published
    property AdapterIndex: Integer read FAdapterIndex write SetAdapterIndex;
    property AdapterName: String read FAdapterName write SetAdapterName;
    property AdapterDescription: String read FAdapterDescription write SetAdapterDescription;
    property CaptureEnabled: Boolean read FCaptureEnabled write FCaptureEnabled;
    property ThreadSafe: Boolean read FThreadSafe write FThreadSafe;
    property Filter: String read FFilter write FFilter;
    property SnapLen: Integer read FSnapLen write FSnapLen;
    property Flags: Integer read FFlags write FFlags;
    property ReadTimeout: Integer read FReadTimeout write FReadTimeout;
  end;

// ----------------------------------------------------------------------------
// TSnoop
// ----------------------------------------------------------------------------
  TSnoop = class(TCustomSnoop)
  protected
    procedure OnCaptureCB; override;
  public
    constructor Create(AOwner: TComponent); override;
  protected
    // property
    FOnCapture: TOnCaptureEvent;
  published
    property OnGetRemoteAdapterInfo;    
    property OnCapture: TOnCaptureEvent read FOnCapture write FOnCapture;
  end;

// ----------------------------------------------------------------------------
// TSnoopIP
// ----------------------------------------------------------------------------
  TSnoopIP = class(TCustomSnoop)
  protected
    procedure OnCaptureCB; override;
  public
    constructor Create(AOwner: TComponent); override;
  protected
    // property
    FOnCaptureIP: TOnCaptureIPEvent;
  published
    property OnGetRemoteAdapterInfo;
    property OnCaptureIP: TOnCaptureIPEvent read FOnCaptureIP write FOnCaptureIP;
  end;

// ----------------------------------------------------------------------------
// TSnoopARP
// ----------------------------------------------------------------------------
  TSnoopARP = class(TCustomSnoop)
  protected
    procedure OnCaptureCB; override;
  public
    constructor Create(AOwner: TComponent); override;
  protected
    // property
    FOnCaptureARP: TOnCaptureARPEvent;
  published
    property OnGetRemoteAdapterInfo;
    property OnCaptureARP: TOnCaptureARPEvent read FOnCaptureARP write FOnCaptureARP;
  end;

// ----------------------------------------------------------------------------
// TSnoopICMP
// ----------------------------------------------------------------------------
  TSnoopICMP = class(TCustomSnoop)
  protected
    procedure OnCaptureCB; override;
  public
    constructor Create(AOwner: TComponent); override;
  protected
    // property
    FOnCaptureICMP: TOnCaptureICMPEvent;
  published
    property OnGetRemoteAdapterInfo;
    property OnCaptureICMP: TOnCaptureICMPEvent read FOnCaptureICMP write FOnCaptureICMP;
  end;

// ----------------------------------------------------------------------------
// TSnoopTCP
// ----------------------------------------------------------------------------
  TSnoopTCP = class(TCustomSnoop)
  protected
    procedure OnCaptureCB; override;
  public
    constructor Create(AOwner: TComponent); override;
  protected
    // property
    FOnCaptureTCP: TOnCaptureTCPEvent;
  published
    property OnGetRemoteAdapterInfo;
    property OnCaptureTCP: TOnCaptureTCPEvent read FOnCaptureTCP write FOnCaptureTCP;
  end;

// ----------------------------------------------------------------------------
// TSnoopTCPData
// ----------------------------------------------------------------------------
  TSnoopTCPData = class(TCustomSnoop)
  protected
    procedure OnCaptureCB; override;
  public
    constructor Create(AOwner: TComponent); override;
  protected
    // property
    FOnCaptureTCPData: TOnCaptureTCPDataEvent;
  published
    property OnGetRemoteAdapterInfo;
    property OnCaptureTCPData: TOnCaptureTCPDataEvent read FOnCaptureTCPData write FOnCaptureTCPData;
  end;

// ----------------------------------------------------------------------------
// TSnoopUDP
// ----------------------------------------------------------------------------
  TSnoopUDP = class(TCustomSnoop)
  protected
    procedure OnCaptureCB; override;
  public
    constructor Create(AOwner: TComponent); override;
  protected
    // property
    FOnCaptureUDP: TOnCaptureUDPEvent;
  published
    property OnGetRemoteAdapterInfo;
    property OnCaptureUDP: TOnCaptureUDPEvent read FOnCaptureUDP write FOnCaptureUDP;
  end;

// ----------------------------------------------------------------------------
// TSnoopUDPData
// ----------------------------------------------------------------------------
  TSnoopUDPData = class(TCustomSnoop)
  protected
    procedure OnCaptureCB; override;
  public
    constructor Create(AOwner: TComponent); override;
  protected
    // property
    FOnCaptureUDPData: TOnCaptureUDPDataEvent;
  published
    property OnGetRemoteAdapterInfo;
    property OnCaptureUDPData: TOnCaptureUDPDataEvent read FOnCaptureUDPData write FOnCaptureUDPData;
  end;

// ----------------------------------------------------------------------------
// TSnoopStatistics
// ----------------------------------------------------------------------------
  TSnoopStatistics = class(TCustomSnoop)
  protected
    procedure OnCaptureCB; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Open: Boolean;
  protected
    // property
    FOnCaptureStatistics: TOnCaptureStatisticsEvent;
  published
    property OnCaptureStatistics: TOnCaptureStatisticsEvent read FOnCaptureStatistics write FOnCaptureStatistics;
  end;

// ----------------------------------------------------------------------------
// TSnoopDump
// ----------------------------------------------------------------------------
  TSnoopDump = class(TComponent)
  protected
    FActive: Boolean;
    FError: String;
    FPcapDumper: ppcap_dumper_t;
  public
    property Active: Boolean read FActive;
    property Error: String read FError;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Open(Pcap: PPcap; FileName: String): Boolean;
    function Close: Boolean;
    procedure Write(PKTHdr: PPCAP_PKTHDR; Data: PChar);
  end;

// ----------------------------------------------------------------------------
// TSnoopMyNetwork
// ----------------------------------------------------------------------------
  TSnoopMyNetwork = class(TComponent)
  protected
    // used internally
    FSnoop: TSnoop;
  public
    // property
    IP: UINT32;
    Mac: MACADDR;
    SubnetMask: UINT32;
    Gateway: UINT32;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh;
  protected
    function GetAdapterIndex: Integer;
    procedure SetAdapterIndex(Value: Integer);
    function GetAdapterName: String;
    function GetAdapterDescription: String;
    procedure SetAdapterName(Value: String);
    procedure SetAdapterDescription(Value: String);
    function GetAdapterNames: TStringList;
    function GetAdapterDescriptions: TStringList;
    function GetStrIP: String;
    function GetStrMac: String;
    function GetStrGateway: String;
    function GetStrSubnetMask: String;
    procedure SetStrIP(Value: String);
    procedure SetStrMac(Value: String);
    procedure SetStrGateway(Value: String);
    procedure SetStrSubnetMask(Value: String);
  public
    property AdapterNames: TStringList read GetAdapterNames;
    property AdapterDescriptions: TStringList read GetAdapterDescriptions;
  published
    property AdapterIndex: Integer read GetAdapterIndex write SetAdapterIndex;
    property AdapterName: String read GetAdapterName write SetAdapterName;
    property AdapterDescription: String read GetAdapterDescription write SetAdapterDescription;
    property StrIP: String read GetStrIP write SetStrIP;
    property StrMac: String read GetStrMac write SetStrMac;
    property StrSubnetMask: String read GetStrSubnetMask write SetStrSubnetMask;
    property StrGateway: String read GetStrGateway write SetStrGateway;
  public
    function IsSameLanIP(IP: UINT32): Boolean;
    function GetAdjIP(IP: UINT32): UINT32;
  end;

// ----------------------------------------------------------------------------
// General Function
// ----------------------------------------------------------------------------
function snoopIsIP(ETHHdr: PETH_HDR; IPHdr: PPIP_HDR = nil): Boolean;
function snoopIsARP(ETHHdr: PETH_HDR; ARPHdr: PPARP_HDR= nil): Boolean;
function snoopIsICMP(IPHdr: PIP_Hdr; ICMPHdr: PPICMP_HDR = nil): Boolean;
function snoopIsTCP(IPHdr: PIP_HDR; TCPHdr: PPTCP_HDR = nil): Boolean;
function snoopIsUDP(IPHdr: PIP_HDR; UDPHdr: PPUDP_HDR = nil): Boolean;
function snoopIsTCPData(IPHdr: PIP_HDR; TCPHdr: PTCP_HDR; TCPData: PPChar = nil; TCPDataLen: PInteger = nil): Boolean;
function snoopIsUDPData(IPHdr: PIP_HDR; UDPHdr: PUDP_HDR; UDPData: PPChar = nil; UDPDataLen: PInteger = nil): Boolean;
function snoopIsTCPOption(TCPHdr: PTCP_HDR; TCPOption: PPChar; TCPOptionLen: PInteger): Boolean;

function snoopMac2Str(Mac: MACADDR): String;
function snoopMac2StrFormat(Mac: MACADDR; Format: String): String; // by gilgil 2003.10.28
function snoopIP2Str(IP: UINT32): String;
function snoopStr2Mac(s: String): MACADDR;
function snoopStr2IP(s: String): UINT32;
function snoopIsCleanMac(Mac: PMACADDR): Boolean; overload;
function snoopCompareMac(Mac1: PMACADDR; Mac2: PMACADDR): Boolean;

function snoopIPChecksum(IPHdr: PIP_HDR): UINT16;
function snoopTCPChecksum(IPHdr: PIP_HDR; TCPHdr: PTCP_HDR): UINT16;
function snoopUDPChecksum(IPHdr: PIP_HDR; UDPHdr: PUDP_HDR): UINT16;
function snoopRecalculateChecksum(OldChecksum: UINT16; OldValue: UINT16; NewValue: UINT16): UINT16;
function snoopSendPacket(Pcap: PPcap; Buffer: PChar; Size: Integer): Integer;

procedure snoopSendARP(Pcap: PPcap;
  ETHHdr: PETH_HDR;
  ARPHdr: PARP_HDR);
procedure snoopSendTCPBlock(Pcap: PPcap;
  MyMac: PMACADDR;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR);
procedure snoopSendHTTPBlock(Pcap: PPcap;
  MyMac: PMACADDR;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR;
  TCPData: PChar;
  TCPDataLen: Integer;
  BlockHTML: PChar;
  BlockHTMLLen: Integer);

// ----------------------------------------------------------------------------
// Global Variables
// ----------------------------------------------------------------------------
var
 snoopBroadcastMac: MACADDR = ( $FF, $FF, $FF, $FF, $FF, $FF );
 snoopCleanMac:     MACADDR = ( $00, $00, $00, $00, $00, $00 );

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Snoop', [
    TSnoop,
    TSnoopIP,
    TSnoopARP,
    TSnoopICMP,
    TSnoopTCP,
    TSnoopUDP,
    TSnoopTCPData,
    TSnoopUDPData,
    TSnoopStatistics,
    TSnoopDump,
    TSnoopMyNetwork]);
end;

// ----------------------------------------------------------------------------
// TSnoopThread
// ----------------------------------------------------------------------------
constructor TSnoopThread.Create(Snoop: TCustomSnoop);
begin
  inherited Create;
  FSnoop := Snoop;
end;

destructor TSnoopThread.Destroy;
begin
  inherited;
end;

function TSnoopThread.Open: Boolean;
begin
  Result := inherited Open;
end;

function TSnoopThread.Close: Boolean;
begin
  Result := inherited Close;
  VDWaitFor;
end;

procedure TSnoopThread.Run;
var
  i: Integer;
  Snoop: TCustomSnoop;
  Pcap: PPcap;
  ThreadSafe: Boolean;
  PKTHdr: PPCAP_PKTHDR;
  PacketData: PETH_HDR;
begin
  Snoop := FSnoop; // use local variable for execute optimization
  if Snoop = nil then
    exit;
  Pcap := Snoop.Pcap;
  ThreadSafe := Snoop.ThreadSafe;
  while not Terminated do
  begin
    // GTrace('[Snoop.pas] TSnoopThread.Run bef pcap_next_ex'); // by gilgil 2006.10.24
    i := pcap_next_ex(Pcap, @PKTHdr, @PacketData);
    // GTrace('[Snoop.pas] TSnoopThread.Run aft pcap_next_ex %d', [i]); // by gilgil 2006.10.24
    if Terminated then break;
    case i of
      -1: begin // if an error occurred
        Snoop.FError := 'Snoop Error: pcap_next_ex return -1';
        GTrace('[Snoop.pas] TSnoopThread.Run pcap_next_ex return -1 (%s)', [PChar(Snoop.FError)]);
        break;
      end;
      0: begin
        continue; // if the timeout set with pcap_open_live() has elapsed.
      end;
    end;
    Snoop.FPKTHdr := PKTHdr;
    Snoop.FETHHdr := PacketData;
    // GTrace('[Snoop.pas] TSnoopThread.Run bef OnCaptureCB'); // by gilgil 2007.01.14
    if ThreadSafe then
    begin
      Synchronize(Snoop.OnCaptureCB);
    end else
    begin
      Snoop.OnCaptureCB;
    end;
    // GTrace('[Snoop.pas] TSnoopThread.Run aft OnCaptureCB'); // by gilgil 2007.01.14
  end;
end;

// ----------------------------------------------------------------------------
// TCustomSnoop
// ----------------------------------------------------------------------------
procedure TCustomSnoop.SetAdapterIndex(Value: Integer);
begin
  if Value = FAdapterIndex then
    exit;
  if (Value < 0) or (Value >= FAdapterNames.Count) then
  begin
    FAdapterIndex := -1;
    FAdapterName := '';
  end else
  begin
    FAdapterIndex := Value;
    FAdapterName := FAdapterNames[Value];
    FAdapterDescription := FAdapterDescriptions[Value];
  end;
end;

procedure TCustomSnoop.SetAdapterName(Value: String);
var
  i: Integer;
begin
  if Value = FAdapterName then
    exit;
  for i := 0 to FAdapterNames.Count -1 do
  begin
    if Value = FAdapterNames[i] then
    begin
      FAdapterIndex := i;
      FAdapterName := FAdapterNames[i];
      FAdapterDescription := FAdapterDescriptions[i];
      exit;
    end;
  end;
  FAdapterIndex := -1;
  FAdapterName := '';
  FAdapterDescription := '';
end;

procedure TCustomSnoop.SetAdapterDescription(Value: String);
var
  i: Integer;
begin
  if Value = FAdapterDescription then
    exit;
  for i := 0 to FAdapterDescriptions.Count -1 do
  begin
    if Value = FAdapterDescriptions[i] then
    begin
      FAdapterIndex := i;
      FAdapterName := FAdapterNames[i];
      FAdapterDescription := FAdapterDescriptions[i];
      exit;
    end;
  end;
  FAdapterIndex := -1;
  FAdapterName := '';
  FAdapterDescription := '';
end;

function TCustomSnoop.DoOpen(Source: PChar; Auth: ppcap_rmtauth; Dev: ppcap_if_t): Boolean;
label
  _error;
var
  ErrBuf: array [0 .. PCAP_ERRBUF_SIZE - 1] of Char;
  DataLink: Integer;
begin
  FPcap := pcap_open(Source, FSnapLen, FFlags, FReadTimeout, Auth, ErrBuf);
  if FPcap = nil then
  begin
    FError := ErrBuf;
    GTrace('[Snoop.pas] TCustomSnoop.DoOpen pcap_open return nil (%s)', [PChar(FError)]);
    goto _error;
  end;

  DataLink := pcap_datalink(FPcap);
  if DataLink <> DLT_EN10MB then
  begin
    GTrace('[Snoop.pas] TCustomSnoop.DoOpen pcap_datalink=%d', [DataLink]);
  end;

  FSourceName := String(Source);
  if not ProcessFilter(Dev) then goto _error;
  FActive := true;
  Result := true;
  exit;
_error:
  Result := false;
end;

procedure TCustomSnoop.CreateSnoopThread(AutoResume: Boolean);
begin
  // Start Snoop Read Thread
  FThread := TSnoopThread.Create(Self);
  FThread.AutoResume := AutoResume;
  FThread.FreeOnTerminate := false;
  FThread.Open;
end;

procedure TCustomSnoop.GetAdapterNames;
var
  i: Integer;
  Dev: ppcap_if_t;
  AllDevs: ppcap_if_t;
  ErrBuf: array[0 .. PCAP_ERRBUF_SIZE - 1] of Char;
begin
  if AdapterNames = nil then exit;
  AdapterNames.Capacity := 10;
  i := pcap_findalldevs(@AllDevs, ErrBuf);
  if i <> 0 then // if error occured
  begin
    Error := ErrBuf;
    exit;
  end;
  Dev := AllDevs;
  while Dev <> nil do
  begin
    AdapterNames.Add(Dev^.name);
    Dev := Dev^.next;
  end;
  FError := '';
  pcap_freealldevs(AllDevs);
end;

procedure TCustomSnoop.GetAdapterDescriptions;
var
  i: Integer;
  Dev: ppcap_if_t;
  AllDevs: ppcap_if_t;
  ErrBuf: array[0 .. PCAP_ERRBUF_SIZE - 1] of Char;
begin
  if AdapterDescriptions = nil then exit;
  AdapterDescriptions.Capacity := 10;
  i := pcap_findalldevs(@AllDevs, ErrBuf);
  if i <> 0 then // if error occured
  begin
    Error := ErrBuf;
    exit;
  end;
  Dev := AllDevs;
  while Dev <> nil do
  begin
    AdapterDescriptions.Add(Dev^.description);
    Dev := Dev^.next;
  end;
  FError := '';
  pcap_freealldevs(AllDevs);
end;

function TCustomSnoop.ProcessFilter(Dev: ppcap_if_t): Boolean;
label
  _error;
var
  NetMask: u_int;
  FCode: bpf_program;
  nRes: Integer;
begin
  if (Dev <> nil) and (Dev^.addresses <> nil) then
    NetMask := Dev^.addresses.netmask.sin_addr.S_addr
  else
    NetMask := $FFFFFFFF;
  nRes := pcap_compile(FPcap, @FCode, PChar(FFilter), 1, NetMask);
  if  nRes < 0  then
  begin
    FError := String(pcap_geterr(FPcap));
    GTrace('[Snoop.pas] TCustomSnoop.ProcessFilter pcap_compile return %d (%s)', [nRes, PChar(FError)]);
    goto _error;
  end;
  nRes := pcap_setfilter(FPcap, @FCode);
  if  nRes < 0 then
  begin
    FError := String(pcap_geterr(FPcap));
    GTrace('[Snoop.pas] TCustomSnoop.ProcessFilter pcap_compile %d minus (%s)', [nRes, PChar(FError)]);
    goto _error;
  end;
  Result := true;
  exit;
_error:
  Result := false;
end;

constructor TCustomSnoop.Create(AOwner: TComponent);
begin
  inherited;
  FActive := false;
  FAdapterIndex := -1;
  FAdapterName := '';
  FAdapterNames := TStringList.Create;
  GetAdapterNames;
  FAdapterDescriptions := TStringList.Create;
  GetAdapterDescriptions;
  FFilter := '';
  FFlags := PCAP_OPENFLAG_PROMISCUOUS;
  FReadTimeout := SNOOP_DEFAULT_READTIMEOUT;
  FSnapLen := SNOOP_DEFAULT_SNAPLEN;
  FThreadSafe := true;
  FCaptureEnabled := true;
  FError := '';
  FThread := nil;
  FPcap := nil;
end;

destructor TCustomSnoop.Destroy;
begin
  Close;
  if FAdapterNames <> nil then
  begin
    FAdapterNames.Free;
    FAdapterNames := nil;
  end;
  if FAdapterDescriptions <> nil then
  begin
    FAdapterDescriptions.Free;
    FAdapterDescriptions := nil;
  end;
  inherited;
end;

function TCustomSnoop.Open(AutoResume: Boolean = true): Boolean;
label
  _error;
var
  i: Integer;
  Dev: ppcap_if_t;
  AllDevs: ppcap_if_t;
  ErrBuf: array[0 .. PCAP_ERRBUF_SIZE - 1] of Char;
begin
  if FActive then
  begin
    FError := 'Snoop Error: Already open';
    GTrace('[Snoop.pas] TCustomSnoop.Open (%s)', [PChar(FError)]);
    goto _error;
  end;
  i := pcap_findalldevs(@AllDevs, ErrBuf);
  if i <> 0 then // if error occured
  begin
    Error := ErrBuf;
    goto _error;
  end;
  Dev := AllDevs;
  while Dev <> nil do
  begin
    if StrComp(Dev^.name, PChar(FAdapterName)) = 0 then break;
    Dev := Dev^.next;
  end;
  if Dev = nil then
  begin
    FError := 'Snoop Error: Invalid AdapterName';
    GTrace('[Snoop.pas] TCustomSnoop.Open (%s)', [PChar(FError)]);
    goto _error;
  end else
  begin
    if not DoOpen(PChar(FAdapterName), nil, Dev) then goto _error;
    if FCaptureEnabled then
      CreateSnoopThread(AutoResume);
  end;
  pcap_freealldevs(AllDevs);
  Result := true;
  exit;
_error:
  Result := false;
end;

function TCustomSnoop.Open(Host: String; UserName: String; Password: String): Boolean;
label
  _error;
var
  i: Integer;
  Source: String;
  AdapterIndex: Integer;
  Auth: pcap_rmtauth;
  Dev: ppcap_if_t;
  AllDevs: ppcap_if_t;
  ErrBuf: array[0 .. PCAP_ERRBUF_SIZE - 1] of Char;
  AdapterNames: TStringList;
  AdapterDescriptions: TStringList;
begin
  if FActive then
  begin
    FError := 'Snoop Error: Already open';
    GTrace('[Snoop.pas] TCustomSnoop.Open (%s)', [PChar(FError)]);
    goto _error;
  end;
  Source := 'rpcap://' + Host + '/';
  Auth.type_ := RPCAP_RMTAUTH_PWD;
  Auth.username := PChar(UserName);
  Auth.password := PChar(Password);
  i := pcap_findalldevs_ex(PChar(Source), @Auth, @AllDevs, ErrBuf);
  if i <> 0 then // if error occured
  begin
    Error := ErrBuf;
    goto _error;
  end;
  AdapterNames := TStringList.Create;
  AdapterDescriptions := TStringList.Create;
  Dev := AllDevs;
  while Dev <> nil do
  begin
    AdapterNames.Add(Dev^.name);
    AdapterDescriptions.Add(Dev^.description);
    Dev := Dev^.next;
  end;
  AdapterIndex := -1;
  if @FOnGetRemoteAdapterInfo <> nil then
    FOnGetRemoteAdapterInfo(Self, AdapterNames, AdapterDescriptions, AdapterIndex);
  if AdapterIndex <> -1 then
  Dev := AllDevs;
  for i := 0 to AdapterIndex - 1 do
    Dev := Dev^.next;
  if Dev = nil then
  begin
    FError := 'Snoop Error: Cancel By User or Invalid AdapterName';
    GTrace('[Snoop.pas] TCustomSnoop.Open (%s)', [PChar(FError)]);
    goto _error;
  end else
  begin
    Source := Dev^.name;
    if not DoOpen(PChar(Source), @Auth, Dev) then goto _error;
    if FCaptureEnabled then
      CreateSnoopThread;
  end;
  pcap_freealldevs(AllDevs);
  AdapterNames.Free;
  AdapterDescriptions.Free;
  Result := true;
  exit;
_error:
  Result := false;
end;

function TCustomSnoop.LoadFromFile(FileName: String): Boolean;
label
  _error;
var
  i: Integer;
  PKTHdr: PPCAP_PKTHDR;
  PacketData: PETH_HDR;
begin
  if FActive then
  begin
    FError := 'Snoop Error: Already open';
    GTrace('[Snoop.pas] TCustomSnoop.LoadFromFile (%s)', [PChar(FError)]);
    goto _error;
  end;
  if FileName = '' then
  begin
    FError := 'Snoop Error: Invalid FileName';
    GTrace('[Snoop.pas] TCustomSnoop.LoadFromFile (%s)', [PChar(FError)]);
    goto _error;
  end;
  if not DoOpen(PChar('file://' + FileName), nil, nil) then goto _error;
  if FCaptureEnabled then
  begin
    while true do
    begin
      i := pcap_next_ex(FPcap, @PKTHdr, @PacketData);
      case i of
        -2: break; // EOF was reached reading from an offline capture
        -1: begin// if an error occurred
          FError := 'Snoop Error: pcap_next_ex return -1';
          GTrace('[Snoop.pas] TCustomSnoop.LoadFromFile (%s)', [PChar(FError)]);
          break;
        end;
        0 : begin
          continue; // if the timeout set with pcap_open_live() has elapsed.
        end;
      end;
      FPKTHdr := PKTHdr;
      FETHHdr := PacketData;
      OnCaptureCB;
    end;
  end;
  FActive := false;
  pcap_close(FPcap);
  Result := true;
  exit;
_error:
  Result := false;
end;

function TCustomSnoop.Close: Boolean;
label
  _error;
begin
  if not FActive then
    goto _error;
  if FThread <> nil then
  begin
    FThread.Terminate;
    FThread.Close;
    FThread.Free;
    FThread := nil;
  end;
  pcap_close(FPcap);
  FPcap := nil;
  FActive := false;
  Result := true;
  exit;
_error:
  Result := false;
end;

// ----------------------------------------------------------------------------
// TSnoop
// ----------------------------------------------------------------------------
procedure TSnoop.OnCaptureCB;
begin
  if @FOnCapture = nil then exit;
  FOnCapture(Self, FPKTHdr, FETHHdr);
end;

constructor TSnoop.Create(AOwner: TComponent);
begin
  inherited;
end;

// ----------------------------------------------------------------------------
// TSnoopIP
// ----------------------------------------------------------------------------
procedure TSnoopIP.OnCaptureCB;
var
  IPHdr: PIP_HDR;
begin
  if @FOnCaptureIP = nil then exit;
  if not snoopIsIP(FETHHdr, @IPHdr) then exit;
  FOnCaptureIP(Self, FPKTHdr, FETHHdr, IPHdr);
end;

constructor TSnoopIP.Create(AOwner: TComponent);
begin
  inherited;
  FFilter := 'ip';
end;

// ----------------------------------------------------------------------------
// TSnoopARP
// ----------------------------------------------------------------------------
procedure TSnoopARP.OnCaptureCB;
var
  ARPHdr: PARP_HDR;
begin
  if @FOnCaptureARP = nil then exit;
  if not snoopIsARP(FETHHdr, @ARPHdr) then exit;
  FOnCaptureARP(Self, FPKTHdr, FETHHdr, ARPHdr);
end;

constructor TSnoopARP.Create(AOwner: TComponent);
begin
  inherited;
  FFilter := 'arp';
end;

// ----------------------------------------------------------------------------
// TSnoopICMP
// ----------------------------------------------------------------------------
procedure TSnoopICMP.OnCaptureCB;
var
  IPHdr: PIP_HDR;
  ICMPHdr: PICMP_HDR;
begin
  if @FOnCaptureICMP = nil then exit;
  if not snoopIsIP(FETHHdr, @IPHdr) then exit;
  if not snoopIsICMP(IPHdr, @ICMPHdr) then exit;
  FOnCaptureICMP(Self, FPKTHdr, FETHHdr, IPHdr, ICMPHdr);
end;

constructor TSnoopICMP.Create(AOwner: TComponent);
begin
  inherited;
  FFilter := 'icmp';
end;

// ----------------------------------------------------------------------------
// TSnoopTCP
// ----------------------------------------------------------------------------
procedure TSnoopTCP.OnCaptureCB;
var
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR;
begin
  if @FOnCaptureTCP = nil then exit;
  if not snoopIsIP(FETHHdr, @IPHdr) then exit;
  if not snoopIsTCP(IPHdr, @TCPHdr) then exit;
  FOnCaptureTCP(Self, FPKTHdr, FETHHdr, IPHdr, TCPHdr);
end;

constructor TSnoopTCP.Create(AOwner: TComponent);
begin
  inherited;
  FFilter := 'tcp';
end;

// ----------------------------------------------------------------------------
// TSnoopTCPData
// ----------------------------------------------------------------------------
procedure TSnoopTCPData.OnCaptureCB;
var
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR;
  TCPData: PChar;
  TCPDataLen: Integer;
begin
  if @FOnCaptureTCPData = nil then exit;
  if not snoopIsIP(FETHHdr, @IPHdr) then exit;
  if not snoopIsTCP(IPHdr, @TCPHdr) then exit;
  if not snoopIsTCPData(IPHdr, TCPHdr, @TCPData, @TCPDataLen) then exit;
  FOnCaptureTCPData(Self, FPKTHdr, FETHHdr, IPHdr, TCPHdr, TCPData, TCPDataLen);
end;

// ----------------------------------------------------------------------------
// TSnoopUDP
// ----------------------------------------------------------------------------
procedure TSnoopUDP.OnCaptureCB;
var
  IPHdr: PIP_HDR;
  UDPHdr: PUDP_HDR;
begin
  if @FOnCaptureUDP = nil then exit;
  if not snoopIsIP(FETHHdr, @IPHdr) then exit;
  if not snoopIsUDP(IPHdr, @UDPHdr) then exit;
  FOnCaptureUDP(Self, FPKTHdr, FETHHdr, IPHdr, UDPHdr);
end;

constructor TSnoopUDP.Create(AOwner: TComponent);
begin
  inherited;
  FFilter := 'udp';
end;

constructor TSnoopTCPData.Create(AOwner: TComponent);
begin
  inherited;
  FFilter := 'tcp';
end;

// ----------------------------------------------------------------------------
// TSnoopUDPData
// ----------------------------------------------------------------------------
procedure TSnoopUDPData.OnCaptureCB;
var
  IPHdr: PIP_HDR;
  UDPHdr: PUDP_HDR;
  UDPData: PChar;
  UDPDataLen: Integer;
begin
  if @FOnCaptureUDPData = nil then exit;
  if not snoopIsIP(FETHHdr, @IPHdr) then exit;
  if not snoopIsUDP(IPHdr, @UDPHdr) then exit;
  if not snoopIsUDPData(IPHdr, UDPHdr, @UDPData, @UDPDataLen) then exit;
  FOnCaptureUDPData(Self, FPKTHdr, FETHHdr, IPHdr, UDPHdr, UDPData, UDPDataLen);
end;

constructor TSnoopUDPData.Create(AOwner: TComponent);
begin
  inherited;
  FFilter := 'udp';
end;

// ----------------------------------------------------------------------------
// TSnoopStatistics
// ----------------------------------------------------------------------------
procedure TSnoopStatistics.OnCaptureCB;
var
  Packets: Int64;
  Bytes: Int64;
  p: PInt64;
begin
  if @FOnCaptureStatistics = nil then exit;
  p := PInt64(FETHHdr);
  Packets := p^;
  inc(p);
  Bytes := p^;
  FOnCaptureStatistics(Self, FPKTHdr, Packets, Bytes);
end;

constructor TSnoopStatistics.Create(AOwner: TComponent);
begin
  inherited;
end;

destructor TSnoopStatistics.Destroy;
begin
  inherited;
end;

function TSnoopStatistics.Open: Boolean;
begin
  inherited Open;
  if FActive then
    pcap_setmode(FPcap, MODE_STAT);
  Result := FActive;
end;

// ----------------------------------------------------------------------------
// TSnoopDump
// ----------------------------------------------------------------------------
constructor TSnoopDump.Create(AOwner: TComponent);
begin
  inherited;
  FActive := false;
  FError := '';
  FPcapDumper := nil;
end;

destructor TSnoopDump.Destroy;
begin
  Close;
  inherited;
end;

function TSnoopDump.Open(Pcap: PPcap; FileName: String): Boolean;
label
  _error;
begin
  if FActive then goto _error;
  if FileName= '' then
  begin
    FError := 'Snoop Error: Invalid file name';
    GTrace('[Snoop.pas] TSnoopDump.Open (%s)', [PChar(FError)]);
    goto _error;
  end;
  FPcapDumper := pcap_dump_open(Pcap, PChar(FileName));
  if FPcapDumper = nil then
  begin
    FError := String(pcap_geterr(Pcap));
    GTrace('[Snoop.pas] TSnoopDump.Open (%s)', [PChar(FError)]);
    goto _error;
  end;
  FActive := true;
  Result := true;
  exit;
_error:
  Result := false;
end;

function TSnoopDump.Close: Boolean;
label
  _error;
begin
  if not FActive then goto _error;
  pcap_dump_close(FPcapDumper);
  FActive := false;
  Result := true;
  exit;
_error:
  Result := false;
end;

procedure TSnoopDump.Write(PKTHdr: PPCAP_PKTHDR; Data: PChar);
begin
  pcap_dump(FPcapDumper, Pcap.ppcap_pkthdr(PKTHdr), Data);
end;

// ----------------------------------------------------------------------------
// TSnoopMyNetwork
// ----------------------------------------------------------------------------

constructor TSnoopMyNetwork.Create(AOwner: TComponent);
begin
  inherited;
  FSnoop := TSnoop.Create(nil);
  Refresh;
end;

destructor TSnoopMyNetwork.Destroy;
begin
  if FSnoop <> nil then
  begin
    FSnoop.Free;
    FSnoop := nil;
  end;
  inherited;
end;

procedure TSnoopMyNetwork.Refresh;
var
  i: Integer;
  p, pAdapterInfo: PIP_ADAPTER_INFO;
  uOutBufLen: ULONG;
  dwRes: DWORD;
begin
  if FSnoop = nil then
  begin
    exit;
  end;
  IP := 0;
  Mac := snoopCleanMac;
  SubnetMask := 0;
  Gateway := 0;
  if FSnoop.FAdapterName = '' then
  begin
    exit;
  end;
  pAdapterInfo := nil;
  uOutBufLen := 0;
  dwRes := GetAdaptersInfo(pAdapterInfo, uOutBufLen);
  if dwRes = ERROR_BUFFER_OVERFLOW then
  begin
    GetMem(pAdapterInfo, uOutBufLen);
    dwRes := GetAdaptersInfo(pAdapterInfo, uOutBufLen);
  end;
  if dwRes <> ERROR_SUCCESS then
  begin
    exit;
  end;
  p := pAdapterInfo;
  while p <> nil do
  begin
    if Pos(String(p^.AdapterName), FSnoop.FAdapterName) <> 0 then
      break;
    p := p^.Next;
  end;
  if p <> nil then
  begin
    if p^.IpAddressList.IpAddress.S[0] <> #0 then
      IP := snoopStr2IP(p^.IpAddressList.IpAddress.S);
    for i := 0 to SNOOP_MAC_SIZE - 1 do
      Mac[i] := p^.Address[i];
    if p^.IpAddressList.IpMask.S[0] <> #0 then
      SubnetMask := snoopStr2IP(p^.IpAddressList.IpMask.S);
    if p^.GatewayList.IpAddress.S[0] <> #0 then
      Gateway := snoopStr2IP(p^.GatewayList.IpAddress.S);
  end;
  if pAdapterInfo <> nil then FreeMem(pAdapterInfo);
end;

function TSnoopMyNetwork.GetAdapterIndex: Integer;
begin
  if FSnoop = nil then
  begin
    Result := -1;
    exit;
  end;
  Result := FSnoop.FAdapterIndex;
end;

procedure TSnoopMyNetwork.SetAdapterIndex(Value: Integer);
var
  OldAdapterIndex: Integer;
begin
  if FSnoop = nil then
  begin
    exit;
  end;
  OldAdapterIndex := FSnoop.FAdapterIndex;
  FSnoop.SetAdapterIndex(Value);
  if FSnoop.FAdapterIndex = OldAdapterIndex then exit;
  Refresh;
end;

function TSnoopMyNetwork.GetAdapterName: String;
begin
  if FSnoop = nil then
  begin
    Result := '';
    exit;
  end;
  Result := FSnoop.FAdapterName;
end;

function TSnoopMyNetwork.GetAdapterDescription: String;
begin
  if FSnoop = nil then
  begin
    Result := '';
    exit;
  end;
  Result := FSnoop.FAdapterDescription;
end;

procedure TSnoopMyNetwork.SetAdapterName(Value: String);
begin
  if FSnoop = nil then
  begin
    exit;
  end;
  FSnoop.SetAdapterName(Value);
  Refresh;
end;

procedure TSnoopMyNetwork.SetAdapterDescription(Value: String);
begin
  if FSnoop = nil then
  begin
    exit;
  end;
  FSnoop.SetAdapterDescription(Value);
  Refresh;
end;

function TSnoopMyNetwork.GetAdapterNames: TStringList;
begin
  if FSnoop = nil then
  begin
    Result := nil;
    exit;
  end;
  Result := FSnoop.FAdapterNames;
end;

function TSnoopMyNetwork.GetAdapterDescriptions: TStringList;
begin
  if FSnoop = nil then
  begin
    Result := nil;
    exit;
  end;
  Result := FSnoop.FAdapterDescriptions;
end;

function TSnoopMyNetwork.GetStrIP: String;
begin
  Result := snoopIP2Str(IP);
end;

function TSnoopMyNetwork.GetStrMac: String;
begin
  Result := snoopMac2Str(Mac);
end;

function TSnoopMyNetwork.GetStrGateway: String;
begin
  Result := snoopIP2Str(Gateway);
end;

function TSnoopMyNetwork.GetStrSubnetMask: String;
begin
  Result := snoopIP2Str(SubnetMask);
end;

procedure TSnoopMyNetwork.SetStrIP(Value: String);
begin
    // do nothing
end;

procedure TSnoopMyNetwork.SetStrMac(Value: String);
begin
    // do nothing
end;

procedure TSnoopMyNetwork.SetStrGateway(Value: String);
begin
    // do nothing
end;

procedure TSnoopMyNetwork.SetStrSubnetMask(Value: String);
begin
    // do nothing
end;

function TSnoopMyNetwork.IsSameLanIP(IP: UINT32): Boolean;
begin
  Result := (Self.IP and Self.SubnetMask) = (IP and Self.SubnetMask);
end;

function TSnoopMyNetwork.GetAdjIP(IP: UINT32): UINT32;
begin
  if IsSameLanIP(IP) then
    Result := IP
  else
    Result := Self.Gateway;
end;

// ----------------------------------------------------------------------------
// General Function
// ----------------------------------------------------------------------------
function snoopIsIP(ETHHdr: PETH_HDR; IPHdr: PPIP_HDR): Boolean;
begin
  Result := false;
  if ntohs(ETHHdr.Protocol) <> PROTO_IP then
    exit;
  if IPHdr <> nil then
    IPHdr^ := PIP_HDR(UINT32(ETHHdr) + sizeof(ETH_HDR));
  Result := true;
end;

function snoopIsARP(ETHHdr: PETH_HDR; ARPHdr: PPARP_HDR): Boolean;
begin
  Result := false;
  if ntohs(ETHHdr.Protocol) <> PROTO_ARP then
    exit;
  if ARPHdr <> nil then
    ARPHdr^ := PARP_HDR(UINT32(ETHHdr) + sizeof(ETH_HDR));
  Result := true;
end;

function snoopIsICMP(IPHdr: PIP_Hdr; ICMPHdr: PPICMP_HDR): Boolean;
begin
  Result := false;
  if IPHdr.Protocol <> PROTO_ICMP then
    exit;
  if ICMPHdr <> nil then
    ICMPHdr^ := PICMP_HDR(UINT32(IPHdr) + sizeof(IP_HDR));
  Result := true;
end;

function snoopIsTCP(IPHdr: PIP_HDR; TCPHdr: PPTCP_HDR): Boolean;
begin
  Result := false;
  if IPHdr.Protocol <> PROTO_TCP then
    exit;
  if TCPHdr <> nil then
    TCPHdr^ := PTCP_HDR(UINT32(IPHdr) + sizeof(IP_HDR));
  Result := true;
end;

function snoopIsUDP(IPHdr: PIP_HDR; UDPHdr: PPUDP_HDR): Boolean;
begin
  Result := false;
  if IPHdr.Protocol <> PROTO_UDP then exit;
  if UDPHdr <> nil then
    UDPHdr^ := PUDP_HDR(UINT32(IPHdr) + sizeof(TCP_HDR));
  Result := true;
end;

function snoopIsTCPData(IPHdr: PIP_HDR; TCPHdr: PTCP_HDR; TCPData: PPChar; TCPDataLen: PInteger): Boolean;
var
  _TCPHdrLen: Integer;
  _Data: PChar;
  _Len: Integer;
begin
  _TCPHdrLen := ((TCPHdr.Off_Rsvd and $F0) shr 4) * sizeof(UINT32);
  _Data := PChar(UINT32(TCPHdr) + UINT32(_TCPHdrLen));
  _Len := ntohs(IPHdr.Len) - sizeof(TCP_HDR) - _TCPHdrLen;
  if TCPData <> nil then TCPData^ := _Data;
  if TCPDataLen <> nil then TCPDataLen^ := _Len;
  Result := _Len > 0;
end;

function snoopIsUDPData(IPHdr: PIP_HDR; UDPHdr: PUDP_HDR; UDPData: PPChar; UDPDataLen: PInteger): Boolean;
var
  _UDPDataLen: Integer;
begin
  if UDPData <> nil then
    UDPData^ := PChar(UINT32(UDPHdr) + sizeof(UDP_HDR));
  _UDPDataLen := ntohs(UDPHdr.Len) - sizeof(UDP_HDR);
  if UDPDataLen <> nil then
    UDPDataLen^ := _UDPDataLen;
  Result := _UDPDataLen > 0;
end;

function snoopIsTCPOption(TCPHdr: PTCP_HDR; TCPOption: PPChar; TCPOptionLen: PInteger): Boolean;
var
  _TCPHdrLen: Integer;
begin
  _TCPHdrLen := ((TCPHdr.Off_Rsvd and $F0) shr 4) * sizeof(UINT32);
  TCPOption^ := PChar(TCPHdr) + sizeof(TCP_HDR);
  TCPOptionLen^ := _TCPHdrLen - sizeof(TCP_HDR);
  Result := TCPOptionLen^ > 0;
end;

function snoopMac2Str(Mac: MACADDR): String;
var
  ch1, ch2: Byte;
  i: Integer;
begin
  Result := '';
  for i := 0 to SNOOP_MAC_SIZE - 1 do
  begin
    ch1 := Mac[i] and $F0;
    ch1 := ch1 shr 4;
    if ch1 > 9 then
      ch1 := ch1 + Ord('A') - 10
    else
      ch1 := ch1 + Ord('0');
    ch2 := Mac[i] and $0F;
    if ch2 > 9 then
      ch2 := ch2 + Ord('A') - 10
    else
      ch2 := ch2 + Ord('0');
    Result := Result + Chr(ch1) + Chr(ch2);
    if i = 2 then
      Result := Result + '-';
  end;
end;

function snoopMac2StrFormat(Mac: MACADDR; Format: String): String;
begin
  Result := SysUtils.Format(Format, [
    Mac[0],
    Mac[1],
    Mac[2],
    Mac[3],
    Mac[4],
    Mac[5]
  ]);
end;

function snoopIP2Str(IP: UINT32): String;
begin
  IP := htonl(IP);
  Result := inet_ntoa(in_addr(IP));
end;

function snoopStr2Mac(s: String): MACADDR;
var
  i: Integer;
  p: PChar;
  ch1, ch2: UINT8;
  Mac: MACADDR;
begin
  p := PChar(s);
  for i := 0 to SNOOP_MAC_SIZE - 1 do
  begin
    ch1 := UINT8(p^); inc(p);
    if (ch1 >= UINT8('A')) and (ch1 <= UINT8('F')) then ch1 := ch1 - UINT8('A') + 10
    else if (ch1 >= UINT8('a')) and (ch1 <= UINT8('f')) then ch1 := ch1 - UINT8('f') + 10
    else ch1 := ch1 - UINT8('0');

    ch2 := UINT8(p^); inc(p);
    if (ch2 >= UINT8('A')) and (ch2 <= UINT8('F')) then ch2 := ch2 - UINT8('A') + 10
    else if (ch2 >= UINT8('a')) and (ch2 <= UINT8('f')) then ch2 := ch2 - UINT8('f') + 10
    else ch2 := ch2 - UINT8('0');

    Mac[i] := (ch1 shl 4) + ch2;
    while (p^ = '-') or (p^ = ':') do inc(p);
  end;
  Result := Mac;
end;

function snoopStr2IP(s: String): UINT32;
begin
  Result := ntohl(inet_addr(PChar(s)));
end;

function snoopIsCleanMac(Mac: PMACADDR): Boolean;
var
  i: Integer;
begin
  Result := true;
  for i := 0 to SNOOP_MAC_SIZE - 1 do
    if Mac[i] <> 0 then
    begin
      Result := false;
      exit;
    end;
end;

function snoopCompareMac(Mac1: PMACADDR; Mac2: PMACADDR): Boolean;
var
  i: Integer;
begin
  for i := 0 to SNOOP_MAC_SIZE - 1 do
  begin
    if Mac1[i] <> Mac2[i] then
    begin
      Result := false;
      exit;
    end;
  end;
  Result := true;
end;

//
// All IPHdr field except IPHdr.Checksum
//
function snoopIPChecksum(IPHdr: PIP_HDR): UINT16;
var
  i: Integer;
  Sum: UINT32;
  p: PUINT16;
begin
  Sum := 0;

  // Add IPHdr Buffer as array of UINT16
  p := PUINT16(IPHdr);
  for i := 0 to sizeof(IP_HDR) div 2 - 1 do
  begin
    Sum := Sum + ntohs(p^);
    inc(p);
  end;

  // Decrease Checksum from Sum
  Sum := Sum - ntohs(IPHdr.Checksum);

  // Recalculate Sum
  while (Sum shr 16) > 0 do
    Sum := (Sum and $FFFF) + (Sum shr 16);
  Sum := not Sum;

  Result := Sum;
end;

//
// All TCPHdr field except TCPHdr.Checksum
// IPHdr.Source, IPHdr.Destination, TCPHdrDataLen
//
function snoopTCPChecksum(IPHdr: PIP_HDR; TCPHdr: PTCP_HDR): UINT16;
var
  i: Integer;
  TCPHdrDataLen: Integer;
  Source, Destination: UINT32;
  Sum: UINT32;
  p: PUINT16;
begin
  TCPHdrDataLen := ntohs(IPHdr.Len) - sizeof(IP_HDR);
  Sum := 0;

  // Add TCPHdr and Data Buffer as array of UINT16
  p := PUINT16(TCPHdr);
  for i := 0 to TCPHdrDataLen div 2 - 1 do
  begin
    Sum := Sum + htons(p^);
    inc(p);
  end;

  // if Len is OddNumber, Add Last Data
  if (TCPHdrDataLen div 2) * 2 <> TCPHdrDataLen then
    Sum := Sum + (htons(p^) and $FF00);

  // Decrease Checksum from Sum
  Sum := Sum - ntohs(TCPHdr.Checksum);

  // Add Source Address
  Source := ntohl(IPHdr.Source);
  Sum := Sum + ((Source and $FFFF0000) shr 16) + (Source and $0000FFFF);

  // Add Destination Addres
  Destination := ntohl(IPHdr.Destination);
  Sum := Sum + ((Destination and $FFFF0000) shr 16) + (Destination and $0000FFFF);

  // Add Extra Information
  Sum := Sum + PROTO_TCP + UINT32(TCPHdrDataLen);

  // Recalculate Sum
  while (Sum shr 16) > 0 do
    Sum := (Sum and $FFFF) + (Sum shr 16);
  Sum := not Sum;

  Result := UINT16(Sum);
end;

//
// All UDPHdr field except UDPHdr.Checksum
// IPHdr.Source, IPHdr.Destination, UDPHdrLen
//
function snoopUDPChecksum(IPHdr: PIP_HDR; UDPHdr: PUDP_HDR): UINT16;
var
  i: Integer;
  UDPHdrLen: Integer;
  Source, Destination: UINT32;
  Sum: UINT32;
  p: PUINT16;
begin
  UDPHdrLen := ntohs(UDPHdr.Len);
  Sum := 0;

  // Add UDPHdr and Data Buffer as array of UINT16
  p := PUINT16(UDPHdr);
  for i := 0 to UDPHdrLen div 2 - 1 do
  begin
    Sum := Sum + htons(p^);
    inc(p);
  end;

  // if Len is OddNumber, Add Last Data
  if (UDPHdrLen div 2) * 2 <> UDPHdrLen then
    Sum := Sum + (htons(p^) and $FF00);

  // Decrease Checksum from Sum
  Sum := Sum - ntohs(UDPHdr.Checksum);

  // Add Source Address
  Source := ntohl(IPHdr.Source);
  Sum := Sum + ((Source and $FFFF0000) shr 16) + (Source and $0000FFFF);

  // Add Destination Addres
  Destination := ntohl(IPHdr.Destination);
  Sum := Sum + ((Destination and $FFFF0000) shr 16) + (Destination and $0000FFFF);

  // Add Extra Information
  Sum := Sum + PROTO_UDP + UINT32(UDPHdrLen);

  // Recalculate Sum
  while (Sum shr 16) > 0 do
    Sum := (Sum and $FFFF) + (Sum shr 16);
  Sum := not Sum;

  Result := UINT16(Sum);
end;

//
// Calculate Checksum on condition that only one value(UINT16) is changed.
//
function snoopRecalculateChecksum(OldChecksum: UINT16; OldValue: UINT16; NewValue: UINT16): UINT16;
var
  Sum: UINT32;
begin
  Sum := OldValue + (not NewValue and $FFFF);
  Sum := Sum + OldChecksum;
  Sum := (Sum and $ffff) + (Sum shr 16);
  Result := UINT16(Sum + (Sum shr 16));
end;

function snoopSendPacket(Pcap: PPcap; Buffer: PChar; Size: Integer): Integer;
begin
  Result := pcap_sendpacket(Pcap, Buffer, Size);
end;

// ----------------------------------------------------------------------------
// General Function
// ----------------------------------------------------------------------------
procedure snoopSendARP(Pcap: PPcap;
    ETHHdr: PETH_HDR;
    ARPHdr: PARP_HDR);
const
  ARP_BUFFER_SIZE = sizeof(ETH_HDR) + sizeof(ARP_HDR);
var
  Buffer: array[0 .. ARP_BUFFER_SIZE - 1] of Char;
begin
  StrMove(PChar(@Buffer[0]), PChar(ETHHdr), sizeof(ETH_HDR));
  StrMove(PChar(UINT32(@Buffer[0]) + sizeof(ETH_HDR)), PChar(ARPHdr), sizeof(ARP_HDR));
  snoopSendPacket(Pcap, Buffer, ARP_BUFFER_SIZE);
end;

procedure snoopSendTCPBlock(Pcap: PPcap;
  MyMac: PMACADDR;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR);
const
  TCP_BLOCK_DATA_SIZE = sizeof(ETH_HDR) + sizeof(IP_HDR) + sizeof(TCP_HDR);
var
  MyBuffer: array [0 .. TCP_BLOCK_DATA_SIZE - 1] of Char;
  MyETHHdr: PETH_HDR;
  MyIPHdr: PIP_HDR;
  MyTCPHdr: PTCP_HDR;

  TCPDataLen: Integer;
begin
  // My Hdr
  MyETHHdr := @MyBuffer[0];
  MyIPHdr := PIP_HDR(UINT32(MyETHHdr) + sizeof(ETH_HDR));
  MyTCPHdr := PTCP_HDR(UINT32(MyIPHdr) + sizeof(IP_HDR));

  // Set Ethernet Hdr
  MyETHHdr^.Destination := ETHHdr^.Destination;
  MyETHHdr^.Source := MyMac^;
  MyETHHdr^.Protocol := ETHHdr^.Protocol;

  // Set IP Hdr
  MyIPHdr^.VerLen := IPHdr^.VerLen;
  MyIPHdr^.Service := IPHdr^.Service;
  MyIPHdr^.Len := ntohs(sizeof(IP_HDR) + sizeof(TCP_HDR));
  MyIPHdr^.Ident := htons(ntohs(IPHdr^.Ident) + 1);
  MyIPHdr^.FlagOff := htons(16384);
  MyIPHdr^.TimeLive := 44;
  MyIPHdr^.Protocol := PROTO_TCP;
  MyIPHdr^.Source := IPHdr^.Source;
  MyIPHdr^.Destination := IPHdr^.Destination;

  // Set TCPHdr
  MyTCPHdr^.Source := TCPHdr^.Source;
  MyTCPHdr^.Destination := TCPHdr^.Destination;
  if snoopIsTCPData(IPHdr, TCPHdr, nil, @TCPDataLen) then
  begin
    MyTCPHdr^.Seq := htonl(ntohl(TCPHdr^.Seq) + TCPDataLen);
  end else
  begin
    MyTCPHdr^.Seq := TCPHdr^.Seq;
  end;
  MyTCPHdr^.Ack := TCPHdr^.Ack;
  MyTCPHdr^.Off_Rsvd := $50;
  MyTCPHdr^.Rsvd_Flags := $04; 
  MyTCPHdr^.Window := 0;
  MyTCPHdr^.UrgPoint := 0;

  // Calculate Checksum
  MyTCPHdr^.Checksum := htons(snoopTCPChecksum(MyIPHdr, MyTCPHdr));
  MyIPHdr^.Checksum := htons(snoopIPChecksum(MyIPHdr));

  // SendPacket
  snoopSendPacket(Pcap, @MyBuffer[0], TCP_BLOCK_DATA_SIZE);
end;

procedure snoopSendHTTPBlock(Pcap: PPcap;
  MyMac: PMACADDR;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR;
  TCPData: PChar;
  TCPDataLen: Integer;
  BlockHTML: PChar;
  BlockHTMLLen: Integer);
var
  SendSize: Integer;
  MyBuffer: PChar;
  MyETHHdr: PETH_HDR;
  MyIPHdr: PIP_HDR;
  MyTCPHdr: PTCP_HDR;
  MyTCPData: PChar;
begin
  // ------------------------------------------------------------------------
  // 1. Send RST Packet HTML Server
  // ------------------------------------------------------------------------
  snoopSendTCPBlock(Pcap, MyMac, ETHHdr, IPHdr, TCPHdr);

  // ------------------------------------------------------------------------
  // 2. Send BlockHTML to HTTP Client
  // ------------------------------------------------------------------------
  SendSize := sizeof(ETH_HDR) + sizeof(IP_HDR) + sizeof(TCP_HDR) + BlockHTMLLen;
  GetMem(MyBuffer, SendSize);

  // My Hdr
  MyETHHdr := @MyBuffer[0];
  MyIPHdr := PIP_HDR(UINT32(MyETHHdr) + sizeof(ETH_HDR));
  MyTCPHdr := PTCP_HDR(UINT32(MyIPHdr) + sizeof(IP_HDR));
  MyTCPData := PChar(UINT32(MyTCPHdr) + sizeof(TCP_HDR));

  // Set Ethernet Hdr
  MyETHHdr^.Destination := ETHHdr^.Source;
  MyETHHdr^.Source := MyMac^;
  MyETHHdr^.Protocol := ETHHdr^.Protocol;

  // Set IP Hdr
  MyIPHdr^.VerLen := IPHdr^.VerLen;
  MyIPHdr^.Service := IPHdr^.Service;
  MyIPHdr^.Len := ntohs(sizeof(IP_HDR) + sizeof(TCP_HDR) + BlockHTMLLen);
  MyIPHdr^.Ident := htons(ntohs(IPHdr^.Ident) + 1);
  MyIPHdr^.FlagOff := htons(16384);
  MyIPHdr^.TimeLive := 44;
  MyIPHdr^.Protocol := PROTO_TCP;
  MyIPHdr^.Source := IPHdr^.Destination;
  MyIPHdr^.Destination := IPHdr^.Source;

  // Set TCPHdr
  MyTCPHdr^.Source := TCPHdr^.Destination;
  MyTCPHdr^.Destination := TCPHdr^.Source;
  MyTCPHdr^.Seq := TCPHdr^.Ack;
  MyTCPHdr^.Ack := htonl(ntohl(TCPHdr^.Seq) + TCPDataLen);
  MyTCPHdr^.Off_Rsvd := $50;
  MyTCPHdr^.Rsvd_Flags := TCP_FLAG_ACK or TCP_FLAG_PSH or TCP_FLAG_FIN;
  MyTCPHdr^.Window := htons(65535);
  MyTCPHdr^.UrgPoint := 0;

  // Set TCPData
  StrMove(MyTCPData, BlockHTML, BlockHTMLLen);

  // Calculate Checksum
  MyTCPHdr^.Checksum := htons(snoopTCPChecksum(MyIPHdr, MyTCPHdr));
  MyIPHdr^.Checksum := htons(snoopIPChecksum(MyIPHdr));

  // SendPacket
  snoopSendPacket(Pcap, @MyBuffer[0], SendSize);

  FreeMem(MyBuffer);
end;

end.

