// ----------------------------------------------------------------------------
//
// Snoop Component Suite version 3.0
//
// http://www.netlab.co.kr
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------

unit SnoopMemory;

interface

uses
  SysUtils, Snoop;

type
  PSnoopMemoryNode = ^TSnoopMemoryNode;
  // PInteger = ^Integer; // gilgil temp 2008.01.28
  TSnoopMemoryNode = packed record
    Index: Integer;
    PKTHdr: PCAP_PKTHDR;
    Next: PSnoopMemoryNode;
    Data: array [0 .. 0] of Char;
  end;
  TSnoopMemory = class
  public
    HdrNode: PSnoopMemoryNode;
  protected
    TailNode: PSnoopMemoryNode;
    function GetEmpty: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Assign(Source: TSnoopMemory);
    function Enqueue(PKTHdr: PPCAP_PKTHDR; Data: PChar): PSnoopMemoryNode;
    function GetNext(HdrNode: PSnoopMemoryNode): PSnoopMemoryNode;
    procedure Next;
  public
    property IsEmpty: Boolean read GetEmpty;
  end;

implementation

function TSnoopMemory.GetEmpty: Boolean;
begin
  Result := HdrNode = nil;
end;

constructor TSnoopMemory.Create;
begin
  HdrNode := nil;
  TailNode := nil;
end;

destructor TSnoopMemory.Destroy;
begin
  Clear;
end;

procedure TSnoopMemory.Clear;
var
  TempNode: PSnoopMemoryNode;
begin
  while HdrNode <> nil do
  begin
    TempNode := HdrNode^.Next;
    FreeMem(HdrNode);
    HdrNode := TempNode;
  end;
  TailNode := nil;
end;

procedure TSnoopMemory.Assign(Source: TSnoopMemory);
var
  SourceNode: PSnoopMemoryNode;
begin
  Clear;
  SourceNode := Source.HdrNode;
  while SourceNode <> nil do
  begin
    Enqueue(@SourceNode^.PKTHdr, SourceNode^.Data);
    SourceNode := SourceNode^.Next;
  end;
end;

function TSnoopMemory.Enqueue(PKTHdr: PPCAP_PKTHDR; Data: PChar): PSnoopMemoryNode;
var
  DataLen: Integer;
  TotalLen: Integer;
  Node: PSnoopMemoryNode;
begin
  DataLen := PKTHdr.len;
  TotalLen := sizeof(TSnoopMemoryNode) + DataLen;
  GetMem(Node, TotalLen);
  if Node = nil then
  begin
    Result := nil;
    exit;
  end;
  // Process Index
  if TailNode = nil then
    Node^.Index := 1
  else
    Node^.Index := TailNode^.Index + 1;
  // Process PKTHdr
  StrMove(PChar(@Node^.PKTHdr), PChar(PKTHdr), sizeof(PCAP_PKTHDR));
  // Process Next
  Node^.Next := nil;
  // Process Data
  StrMove(@Node^.Data, Data, DataLen);
  // Process SnoopMemory's HdrNode and TailNode
  if HdrNode = nil then // if First Enqueue
    HdrNode := Node;
  if TailNode <> nil then // if TailNode Exists
    TailNode^.Next := Node;
  TailNode := Node;
  Result := Node;
end;

function TSnoopMemory.GetNext(HdrNode: PSnoopMemoryNode): PSnoopMemoryNode;
begin
  Result := nil;
  if HdrNode <> nil then
    Result := HdrNode^.Next;
end;

procedure TSnoopMemory.Next;
var
  TempNode: PSnoopMemoryNode;
begin
  TempNode := HdrNode^.Next;
  FreeMem(HdrNode);
  HdrNode := TempNode;
end;

end.


