unit FormMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, ActnList, ComCtrls, ToolWin, IniFiles, ShellAPI,
  Common, ImgList, WinSock, Snoop, SnoopMemory, VDTrace, Globals;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    miSave: TMenuItem;
    miSaveAs: TMenuItem;
    N1: TMenuItem;
    miExit: TMenuItem;
    Panel1: TPanel;
    Edit: TMenuItem;
    miOpen: TMenuItem;
    miClose: TMenuItem;
    Help1: TMenuItem;
    miAbout: TMenuItem;
    miHomePage: TMenuItem;
    ActionList1: TActionList;
    ActionSave: TAction;
    ActionSaveAs: TAction;
    ActionExit: TAction;
    ActionOpen: TAction;
    ActionClose: TAction;
    ActionAbout: TAction;
    ActionHomePage: TAction;
    ToolBar1: TToolBar;
    tbOpen: TToolButton;
    tbClose: TToolButton;
    tbClear: TToolButton;
    View1: TMenuItem;
    ActionOption: TAction;
    miOption: TMenuItem;
    ImageList1: TImageList;
    ActionClear: TAction;
    Snoop1: TSnoop;
    Splitter1: TSplitter;
    mmData: TMemo;
    pcView: TPageControl;
    tsETH: TTabSheet;
    tsIP: TTabSheet;
    tsARP: TTabSheet;
    tsICMP: TTabSheet;
    lvETH: TListView;
    tsTCP: TTabSheet;
    tsTCPData: TTabSheet;
    tsUDPData: TTabSheet;
    tsUDP: TTabSheet;
    lvIP: TListView;
    lvARP: TListView;
    lvICMP: TListView;
    lvTCP: TListView;
    lvUDP: TListView;
    lvTCPData: TListView;
    lvUDPData: TListView;
    StatusBar1: TStatusBar;
    ActionOpenFile: TAction;
    miOpenFile: TMenuItem;
    pmPacketField: TPopupMenu;
    miPacketField: TMenuItem;
    ActionPacketField: TAction;
    chkViewHexa: TCheckBox;
    PacketField1: TMenuItem;
    odFile: TOpenDialog;
    sdFile: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ActionOpenExecute(Sender: TObject);
    procedure ActionCloseExecute(Sender: TObject);
    procedure ActionClearExecute(Sender: TObject);
    procedure ActionOptionExecute(Sender: TObject);
    procedure ActionOpenFileExecute(Sender: TObject);
    procedure ActionSaveExecute(Sender: TObject);
    procedure ActionSaveAsExecute(Sender: TObject);
    procedure ActionExitExecute(Sender: TObject);
    procedure ActionAboutExecute(Sender: TObject);
    procedure ActionHomePageExecute(Sender: TObject);
    procedure Snoop1GetRemoteAdapterInfo(Sender: TObject; AdapterNames,
      AdapterDescriptions: TStringList; var AdapterIndex: Integer);
    procedure pcViewChange(Sender: TObject);
    procedure ActionPacketFieldExecute(Sender: TObject);
    procedure lvDataChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure chkViewHexaClick(Sender: TObject);
    procedure Snoop1Capture(Sender: TObject; PKTHdr: PPCAP_PKTHDR;
      ETHHdr: PETH_HDR);
    procedure lvETHData(Sender: TObject; Item: TListItem);
    procedure lvIPData(Sender: TObject; Item: TListItem);
    procedure lvARPData(Sender: TObject; Item: TListItem);
    procedure lvICMPData(Sender: TObject; Item: TListItem);
    procedure lvTCPData1(Sender: TObject; Item: TListItem);
    procedure lvUDPData1(Sender: TObject; Item: TListItem);
    procedure lvTCPDataData(Sender: TObject; Item: TListItem);
    procedure lvUDPDataData(Sender: TObject; Item: TListItem);
  private
    ActiveListViewIndex: Integer;
    function GetActiveListView: TListView;
    function GetActiveList: TList;
  protected
    // for Capture Memory Management;
    SnoopMemory: TSnoopMemory;
    ETHList: TList;
    IPList: TList;
    ARPList: TList;
    ICMPList: TList;
    TCPList: TList;
    UDPList: TList;
    TCPDataList: TList;
    UDPDataList: TList;
    procedure CreateAllList;
    procedure ClearAllList;
    procedure FreeAllList;
    procedure SynchronizeListWithListView;
  protected
    // for File Save
    SnoopDump: TSnoopDump;
    ExistingFileName: String;
    NewFileName: String;
    IsNeedToSave: Boolean;
    procedure ProcessTempFile;
  protected
    // for View Data in mmData.
    LastListItem: TListItem;
    class function IsNormalChar(ch: Char): Boolean;
    procedure ViewHexa(Data: PChar; Len: Integer);    
    procedure ViewData;
    { Public declarations }
  public
    procedure LoadControl;
    procedure SaveControl;
    procedure SetControl;
    procedure InitializeListView;
    procedure ViewPacket;
  end;

var
  MainForm: TMainForm;

implementation

uses FormOption, FormRemoteAdapter, FormOpen, FormAbout, FormPacketField, FormCapturing;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  mmData.Align := alClient;
  InitializeListView;
  mmData.Lines.Clear;

  SnoopMemory := nil;
  CreateAllList;
  SnoopDump := nil;
  ExistingFileName := '';
  NewFileName := '';
  IsNeedToSave := false;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  Common.LoadListColumnWidth(Self, Global.IniFileName);
  LoadControl;
  SetControl;
  LastListItem := nil;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ActionClose.Execute;
  SaveControl;
  Common.SaveListColumnWidth(Self, Global.IniFileName);

  if SnoopMemory <> nil then
  begin
    SnoopMemory.Free;
    SnoopMemory := nil;
  end;
  FreeAllList;
  if SnoopDump <> nil then
  begin
    SnoopDump.Free;
    SnoopDump := nil;
  end;
  ProcessTempFile;
end;

function TMainForm.GetActiveListView: TListView;
begin
  Result := nil;
  case ActiveListViewIndex of
    0: Result := lvETH;
    1: Result := lvIP;
    2: Result := lvARP;
    3: Result := lvICMP;
    4: Result := lvTCP;
    5: Result := lvUDP;
    6: Result := lvTCPData;
    7: Result := lvUDPData;
  end;
end;

function TMainForm.GetActiveList: TList;
begin
  Result := nil;
  case ActiveListViewIndex of
    0: Result := ETHList;
    1: Result := IPList;
    2: Result := ARPList;
    3: Result := ICMPList;
    4: Result := TCPList;
    5: Result := UDPList;
    6: Result := TCPDataList;
    7: Result := UDPDataList;
  end;
end;

procedure TMainForm.CreateAllList;
begin
  if ETHList = nil then ETHList := TList.Create;
  if IPList = nil then IPList := TList.Create;
  if ARPList = nil then ARPList := TList.Create;
  if ICMPList = nil then ICMPList := TList.Create;
  if TCPList = nil then TCPList := TList.Create;
  if UDPList = nil then UDPList := TList.Create;
  if TCPDataList = nil then TCPDataList := TList.Create;
  if UDPDataList = nil then UDPDataList := TList.Create;
end;

procedure TMainForm.ClearAllList;
begin
  ETHList.Clear;
  IPList.Clear;
  ARPList.Clear;
  ICMPList.Clear;
  TCPList.Clear;
  UDPList.Clear;
  TCPDataList.Clear;
  UDPDataList.Clear;
end;

procedure TMainForm.FreeAllList;
begin
  if ETHList <> nil then
  begin
    ETHList.Free;
    ETHList := nil;
  end;
  if IPList <> nil then
  begin
    IPList.Free;
    IPList := nil;
  end;
  if ARPList <> nil then
  begin
    ARPList.Free;
    ARPList := nil;
  end;
  if ICMPList <> nil then
  begin
    ICMPList.Free;
    ICMPList := nil;
  end;
  if TCPList <> nil then
  begin
    TCPList.Free;
    TCPList := nil;
  end;
  if UDPList <> nil then
  begin
    UDPList.Free;
    UDPList := nil;
  end;
  if TCPDataList <> nil then
  begin
    TCPDataList.Free;
    TCPDataList := nil;
  end;
  if UDPDataList <> nil then
  begin
    UDPDataList.Free;
    UDPDataList := nil;
  end;
end;

procedure TMainForm.SynchronizeListWithListView;
begin
  lvETH.Items.Count := ETHList.Count;
  lvIP.Items.Count := IPList.Count;
  lvARP.Items.Count := ARPList.Count;
  lvICMP.Items.Count := ICMPList.Count;
  lvTCP.Items.Count := TCPList.Count;
  lvUDP.Items.Count := UDPList.Count;
  lvTCPData.Items.Count := TCPDataList.Count;
  lvUDPData.Items.Count := UDPDataList.Count;
end;

procedure TMainForm.ProcessTempFile;
begin
  if ExtractFileExt(ExistingFileName) = '.$$$' then
    if FileExists(ExistingFileName) then
      DeleteFile(ExistingFileName);
end;

class function TMainForm.IsNormalChar(ch: Char): Boolean;
begin
  Result := true;
  if (Ord(ch) < 31) or (Ord(ch) >= 128) then Result := false;
end;

procedure TMainForm.ViewHexa(Data: PChar; Len: Integer);
const
  LINE_WIDTH = 16;
var
  i: Integer;
  Ascii, Hexa: String;
  Strings: TStringList;
begin
  Strings := TStringList.Create;
  while Len > 0 do
  begin
    Ascii := '';
    Hexa := ' ';
    for i := 0 to LINE_WIDTH - 1 do
    begin
      if (i <> 0) and (i mod 4 = 0) then
        Hexa := Hexa + '| ';
      Hexa := Hexa + Format('%2x ', [Ord(Data^)]);

      if IsNormalChar(Data^) then
        Ascii := Ascii + Data^
      else
        Ascii := Ascii + '.';
      dec(Len);
      inc(Data);
      if Len <= 0 then break;
    end;
    Ascii := Copy(Ascii + '         ', 1, LINE_WIDTH);
    Strings.Add(Ascii + ' [' + Hexa+ ']');
  end;
  mmData.Lines := Strings;
  Strings.Clear;
end;


procedure TMainForm.ViewData;
var
  ListView: TListView;
  List: TList;
  ListItem: TListItem;
  i: Integer;
  Node: PSnoopMemoryNode;

  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR;
  UDPHdr: PUDP_HDR;
  Data: PChar;
  Len: Integer;

  s: String;
begin
  ListView := GetActiveListView;
  if ListView = nil then exit;
  List := GetActiveList;
  if List = nil then exit;
  ListItem := ListView.Selected;
  if ListItem = nil then exit;
  i := ListItem.Index;
  Data := nil;
  Len := 0;
  Node := PSnoopMemoryNode(List.Items[i]);
  ETHHdr := @Node^.Data[0];
  if (List = TCPList) or (List = TCPDataList) then
  begin
    snoopIsIP(ETHHdr, @IPHdr);
    snoopIsTCP(IPHdr, @TCPHdr);
    snoopIsTCPData(IPHdr, TCPHdr, @Data, @Len);
    inc(Data, Global.TCPDataOffset);
    dec(Len, Global.TCPDataOffset);
  end else
  if (List = UDPList) or (List = UDPDataList) then
  begin
    snoopIsIP(ETHHdr, @IPHdr);
    snoopIsUDP(IPHdr, @UDPHdr);
    snoopIsUDPData(IPHdr, UDPHdr, @Data, @Len);
    inc(Data, Global.UDPDataOffset);
    dec(Len, Global.UDPDataOffset);
  end;
  if (Data = nil) or (Len <= 0) then
  begin
    mmData.Lines.Clear;
    exit;
  end;

  if not chkViewHexa.Checked then
  begin
    SetLength(s, Len);
    StrMove(PChar(s), Data, Len);
    mmData.Lines.Text := s;
  end else
    ViewHexa(Data, Len);
  LastListItem := ListItem;
end;

procedure TMainForm.LoadControl;
begin
  Global.Load;
  // Coordination
  Left := Global.Left;
  Top := Global.Top;
  Width := Global.Width;
  Height := Global.Height;
  WindowState :=  TWindowState(Global.WindowState);
  pcView.Height := Global.ListViewHeight;
  // ViewHexa
  chkViewHexa.Checked := Global.ViewHexa;
  // ListView
  ActiveListViewIndex := Global.ActiveListViewIndex;
  pcView.ActivePageIndex := ActiveListViewIndex;
end;

procedure TMainForm.SaveControl;
begin
  // Coordination
  Global.Left :=  Left;
  Global.Top := Top;
  Global.Width := Width;
  Global.Height := Height;
  Global.WindowState := Integer(WindowState);  
  Global.ListViewHeight := pcView.Height;
  Global.ActiveListViewIndex := pcView.ActivePageIndex;
  // ViewHexa
  Global.ViewHexa := chkViewHexa.Checked;

  Global.Save;
end;

procedure TMainForm.SetControl;
var
  Active: Boolean;
begin
  Active := Snoop1.Active;
  ActionOpen.Enabled := not Active;
  ActionClose.Enabled := Active;
  ActionClear.Enabled := true;
  ActionOption.Enabled := not Active;
  ActionOpen.Enabled := not Active;
  ActionOpen.Enabled := not Active;
  ActionOpenFile.Enabled := not Active;
  ActionSave.Enabled := not Active and IsNeedToSave;
  ActionSaveAs.Enabled := not Active;
  ActionExit.Enabled := not Active;
  ActionAbout.Enabled := true;
  ActionHomePage.Enabled := true;
  ActionPacketField.Enabled := true;
end;

procedure TMainForm.InitializeListView;
var
  i: Integer;
  ListView: TListView;
begin
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TListView then
    begin
      ListView := Components[i] as TListView;
      ListView.RowSelect := true;
      ListView.ViewStyle := vsReport;
      ListView.Align := alClient;
      ListView.OwnerData := true;
    end;
  end;
  // ETH
  Common.AddListViewField(lvETH, ETH_FIELD, HDR_FIELD);
  Common.AddListViewField(lvETH, ETH_FIELD, ETH_FIELD);
  // IP
  Common.AddListViewField(lvIP, IP_FIELD, HDR_FIELD);
  Common.AddListViewField(lvIP, IP_FIELD, ETH_FIELD);
  Common.AddListViewField(lvIP, IP_FIELD, IP_FIELD);
  // ARP
  Common.AddListViewField(lvARP, ARP_FIELD, HDR_FIELD);
  Common.AddListViewField(lvARP, ARP_FIELD, ETH_FIELD);
  Common.AddListViewField(lvARP, ARP_FIELD, ARP_FIELD);
  // ICMP
  Common.AddListViewField(lvICMP, ICMP_FIELD, HDR_FIELD);
  Common.AddListViewField(lvICMP, ICMP_FIELD, ETH_FIELD);
  Common.AddListViewField(lvICMP, ICMP_FIELD, IP_FIELD);
  Common.AddListViewField(lvICMP, ICMP_FIELD, ICMP_FIELD);
  // TCP
  Common.AddListViewField(lvTCP, TCP_FIELD, HDR_FIELD);
  Common.AddListViewField(lvTCP, TCP_FIELD, ETH_FIELD);
  Common.AddListViewField(lvTCP, TCP_FIELD, IP_FIELD);
  Common.AddListViewField(lvTCP, TCP_FIELD, TCP_FIELD);
  Common.AddListViewField(lvTCP, TCP_FIELD, TCPDATA_FIELD);
  // UDP
  Common.AddListViewField(lvUDP, UDP_FIELD, HDR_FIELD);
  Common.AddListViewField(lvUDP, UDP_FIELD, ETH_FIELD);
  Common.AddListViewField(lvUDP, UDP_FIELD, IP_FIELD);
  Common.AddListViewField(lvUDP, UDP_FIELD, UDP_FIELD);
  Common.AddListViewField(lvUDP, UDP_FIELD, UDPDATA_FIELD);
  // TCPData
  Common.AddListViewField(lvTCPData, TCPDATA_FIELD, HDR_FIELD);
  Common.AddListViewField(lvTCPData, TCPDATA_FIELD, ETH_FIELD);
  Common.AddListViewField(lvTCPData, TCPDATA_FIELD, IP_FIELD);
  Common.AddListViewField(lvTCPData, TCPDATA_FIELD, TCP_FIELD);
  Common.AddListViewField(lvTCPData, TCPDATA_FIELD, TCPDATA_FIELD);
  // UDPData
  Common.AddListViewField(lvUDPData, UDPDATA_FIELD, HDR_FIELD);
  Common.AddListViewField(lvUDPData, UDPDATA_FIELD, ETH_FIELD);
  Common.AddListViewField(lvUDPData, UDPDATA_FIELD, IP_FIELD);
  Common.AddListViewField(lvUDPData, UDPDATA_FIELD, UDP_FIELD);
  Common.AddListViewField(lvUDPData, UDPDATA_FIELD, UDPDATA_FIELD);
end;

procedure TMainForm.ViewPacket;
var
  ListView: TListView;
  List: TList;
begin
  ListView := GetActiveListView;
  List := GetActiveList;
  if ListView.Items.Count <> List.Count then
    ListView.Items.Count := List.Count;
end;

procedure TMainForm.ActionOpenExecute(Sender: TObject);
var
  FileSucceed: Boolean;
begin
  if OpenForm = nil then
    OpenForm := TOpenForm.Create(Application);
  OpenForm.ShowModal;
  if OpenForm.ModalResult <> mrOk then exit;
  Snoop1.Filter := Global.Filter;
  Snoop1.Flags := Global.Flags;
  Snoop1.ReadTimeout := Global.ReadTimeout;
  Snoop1.SnapLen := Global.SnapLen;
  Snoop1.ThreadSafe := Global.ThreadSafe;
  if SnoopMemory = nil then SnoopMemory := TSnoopMemory.Create;
  ActionClear.Execute;
  if Global.OpenType = 2 then // File Capture
  begin
    FileSucceed := Snoop1.LoadFromFile(Global.FileName);
    lvTCPData.Items.Count := TCPDataList.Count;
    if FileSucceed then
    begin
      ExistingFileName := Global.FileName;
      NewFileName := Global.FileName;
    end else
      Application.MessageBox(PChar(Snoop1.Error), 'Error', MB_ICONERROR);
  end else // Adapter Capture or Remote Capture
  begin
    if Global.OpenType = 0 then // Adapter Capture
    begin
      Snoop1.AdapterIndex := Global.AdapterIndex;
      Snoop1.Open;
    end else // Remote Capture
      Snoop1.Open(Global.Host, Global.UserName, Global.Password);
    if Snoop1.Active then
    begin
      if CapturingForm = nil then
        CapturingForm := TCapturingForm.Create(Application);
      CapturingForm.Initialize;
      CapturingForm.Show;

      if SnoopDump = nil then
        SnoopDump := TSnoopDump.Create(nil);
      SnoopDump.Close;
      ExistingFileName := ExtractFilePath(Application.ExeName) + 'temp.$$$';
      NewFileName := '';
      SnoopDump.Open(Snoop1.Pcap, ExistingFileName);
      if not SnoopDump.Active then
      begin
        Application.MessageBox(PChar(Snoop1.Error), 'Error', MB_ICONERROR);
        CapturingForm.Close;
        exit;
      end;
    end else // if fail to open
    begin
      if CapturingForm <> nil then
        if CapturingForm.Visible then
          CapturingForm.Close;
      Application.MessageBox(PChar(Snoop1.Error), 'Error', MB_ICONERROR);
    end;
  end;
  StatusBar1.Panels[0].Text := Snoop1.SourceName;
  SetControl;
end;

procedure TMainForm.ActionCloseExecute(Sender: TObject);
begin
  Snoop1.Close;
  if CapturingForm.Visible then
    CapturingForm.Close;
  if not Global.ThreadSafe then
    SynchronizeListWithListView;
  SetControl;
end;

procedure TMainForm.ActionClearExecute(Sender: TObject);
var
  ListView: TListView;
begin
  SnoopMemory.Clear;
  ClearAllList;
  if CapturingForm <> nil then
  begin
    CapturingForm.Initialize;
    CapturingForm.View;
  end;
  SynchronizeListWithListView;
  ListView := GetActiveListView;
  if ListView = nil then exit;
  ListView.Items.BeginUpdate;
  ListView.Items.EndUpdate;
  mmData.Lines.Clear;
end;

procedure TMainForm.ActionOptionExecute(Sender: TObject);
begin
  if OptionForm = nil then
    OptionForm := TOptionForm.Create(Application);
  OptionForm.ShowModal;
  lvTCPData.Repaint; // Apply changed TCPData Offset
  lvUDPData.Repaint; // Apply changed UDPData Offset
end;

procedure TMainForm.ActionOpenFileExecute(Sender: TObject);
begin
  if odFile.Execute then
  begin
    Global.FileName := odFile.FileName;
    Global.OpenType := 2; // File Capture
    Global.Save;
    ActionOpen.Execute;
  end;
end;

procedure TMainForm.ActionSaveExecute(Sender: TObject);
var
  _SnoopMemory: TSnoopMemory;
  Node: PSnoopMemoryNode;
  Succeed: Boolean;
begin
  if NewFileName = '' then
  begin
    if not sdFile.Execute then exit;
    NewFileName := sdFile.FileName;
  end;
  if SnoopDump.Active then // if need to save
  begin
    _SnoopMemory := TSnoopMemory.Create;
    _SnoopMemory.Assign(SnoopMemory);
    Node := _SnoopMemory.HdrNode;
    while Node <> nil do
    begin
      SnoopDump.Write(@Node^.PKTHdr, Node^.Data);
      Node := _SnoopMemory.GetNext(Node);
    end;
    SnoopDump.Close;
    _SnoopMemory.Free;
  end;

  if ExtractFileExt(ExistingFileName) = '.$$$' then
  begin
    if FileExists(NewFileName) then
      DeleteFile(NewFileName);
    Succeed := MoveFile(PChar(ExistingFileName), PChar(NewFileName))
  end else
    Succeed := CopyFile(PChar(ExistingFileName), PChar(NewFileName), false);
  if Succeed then
  begin
    ExistingFileName := NewFileName;
    IsNeedToSave := false;
  end else
  begin
    Application.MessageBox(PChar('Error $' + IntToHex(GetLastError, 4)),
      'Error', MB_ICONERROR);
  end;
  SetControl;
end;

procedure TMainForm.ActionSaveAsExecute(Sender: TObject);
begin
  if sdFile.Execute then
  begin
    if NewFileName = sdFile.FileName then
    begin
      Application.MessageBox('Same file name', 'Error', MB_ICONERROR);
      exit;
    end;
    NewFileName := sdFile.FileName;
    ActionSave.Enabled := true;
    ActionSave.Execute;
  end;
end;

procedure TMainForm.ActionExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ActionAboutExecute(Sender: TObject);
begin
  if AboutForm = nil then
    AboutForm := TAboutForm.Create(Application);
  AboutForm.ShowModal;
end;

procedure TMainForm.ActionHomePageExecute(Sender: TObject);
begin
  ShellExecute(0, 'open', 'http://www.netlab.co.kr', nil, nil, SW_SHOW);
end;

procedure TMainForm.ActionPacketFieldExecute(Sender: TObject);
var
  ListView: TListView;
begin
  ListView := GetActiveListView;
  if ListView = nil then exit;
  if PacketFieldForm = nil then
    PacketFieldForm := TPacketFieldForm.Create(Application);
  PacketFieldForm.ListView := ListView;
  PacketFieldForm.ShowModal;
end;

procedure TMainForm.Snoop1GetRemoteAdapterInfo(Sender: TObject;
  AdapterNames, AdapterDescriptions: TStringList;
  var AdapterIndex: Integer);
var
  Index: Integer;
begin
  if Global.DontShowJustOne and (AdapterNames.Count = 1) then
  begin
    AdapterIndex := 0;
    exit;
  end;
  if RemoteAdapterForm = nil then
    RemoteAdapterForm := TRemoteAdapterForm.Create(Application);
  RemoteAdapterForm.lbRemoteAdapter.Items := AdapterDescriptions;
  RemoteAdapterForm.ShowModal;
  if RemoteAdapterForm.ModalResult = mrOk then
  begin
    Index := RemoteAdapterForm.lbRemoteAdapter.ItemIndex;
    if Index >= 0 then
      AdapterIndex := Index;
  end;
end;

procedure TMainForm.pcViewChange(Sender: TObject);
begin
  ActiveListViewIndex := pcView.ActivePageIndex;
  SynchronizeListWithListView;
end;

procedure TMainForm.lvDataChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  ViewData;
end;

procedure TMainForm.chkViewHexaClick(Sender: TObject);
begin
  LastListItem := nil;
  ViewData;
end;

procedure TMainForm.Snoop1Capture(Sender: TObject;
  PKTHdr: PPCAP_PKTHDR; ETHHdr: PETH_HDR);
var
  Node: PSnoopMemoryNode;
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR;
  UDPHdr: PUDP_HDR;
begin
  IsNeedToSave := true;
  if Global.OpenType <> 2 then // if not File Capture
    CapturingForm.Process(PKTHdr, ETHHdr);
  Node := SnoopMemory.Enqueue(PKTHdr, PChar(ETHHdr));
  ETHList.Add(Node);
  if snoopIsIP(ETHHdr, @IPHdr) then
  begin
    IPList.Add(Node);
    if snoopIsTCP(IPHdr, @TCPHdr) then
    begin
      TCPList.Add(Node);
      if snoopIsTCPData(IPHdr, TCPHdr) then
        TCPDataList.Add(Node);
    end else
    if snoopIsUDP(IPHdr, @UDPHdr) then
    begin
      UDPList.Add(Node);
      if snoopIsUDPData(IPHdr, UDPHdr) then
        UDPDataList.Add(Node);
    end else
    if snoopIsICMP(IPHdr) then
      ICMPList.Add(Node);
  end else
  if snoopIsARP(ETHHdr) then
    ARPList.Add(Node);
  if Global.ThreadSafe then
    ViewPacket;
end;

procedure TMainForm.lvETHData(Sender: TObject; Item: TListItem);
var
  i: Integer;
  Node: PSnoopMemoryNode;
  ETHHdr: PETH_HDR;
begin
  i := Item.Index;
  if i >= ETHList.Count then exit;
  Node := PSnoopMemoryNode(ETHList.Items[i]);
  ETHHdr := @Node^.Data[0];

  Common.AddPKTHdr(Item, Node);
  Common.AddETH(Item, ETHHdr);
end;

procedure TMainForm.lvIPData(Sender: TObject; Item: TListItem);
var
  i: Integer;
  Node: PSnoopMemoryNode;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
begin
  i := Item.Index;
  if i >= IPList.Count then exit;
  Node := PSnoopMemoryNode(IPList.Items[i]);
  ETHHdr := @Node^.Data[0];

  if not snoopIsIP(ETHHdr, @IPHdr) then exit;

  Common.AddPKTHdr(Item, Node);
  Common.AddETH(Item, ETHHdr);
  Common.AddIP(Item, IPHdr);
end;

procedure TMainForm.lvARPData(Sender: TObject; Item: TListItem);
var
  i: Integer;
  Node: PSnoopMemoryNode;
  ETHHdr: PETH_HDR;
  ARPHdr: PARP_HDR;
begin
  i := Item.Index;
  if i >= ARPList.Count then exit;
  Node := PSnoopMemoryNode(ARPList.Items[i]);
  ETHHdr := @Node^.Data[0];

  if not snoopIsARP(ETHHdr, @ARPHdr) then exit;

  Common.AddPKTHdr(Item, Node);
  Common.AddETH(Item, ETHHdr);
  Common.AddARP(Item, ARPHdr);
end;

procedure TMainForm.lvICMPData(Sender: TObject; Item: TListItem);
var
  i: Integer;
  Node: PSnoopMemoryNode;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  ICMPHdr: PICMP_HDR;
begin
  i := Item.Index;
  if i >= ICMPList.Count then exit;
  Node := PSnoopMemoryNode(ICMPList.Items[i]);
  ETHHdr := @Node^.Data[0];

  if not snoopIsIP(ETHHdr, @IPHdr) then exit;
  if not snoopIsICMP(IPHdr, @ICMPHdr) then exit;

  Common.AddPKTHdr(Item, Node);
  Common.AddETH(Item, ETHHdr);
  Common.AddIP(Item, IPHdr);
  Common.AddICMP(Item, ICMPHdr);
end;

procedure TMainForm.lvTCPData1(Sender: TObject; Item: TListItem);
var
  i: Integer;
  Node: PSnoopMemoryNode;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR;
  TCPData: PChar;
  TCPDataLen: Integer;
  CheckDataLen: Integer;
begin
  i := Item.Index;
  if i >= TCPList.Count then exit;
  Node := PSnoopMemoryNode(TCPList.Items[i]);
  ETHHdr := @Node^.Data[0];

  if not snoopIsIP(ETHHdr, @IPHdr) then exit;
  if not snoopIsTCP(IPHdr, @TCPHdr) then exit;
  snoopIsTCPData(IPHdr, TCPHdr, @TCPData, @TCPDataLen);
  CheckDataLen := ntohs(IPHdr^.Len) - sizeof(IP_HDR) - sizeof(TCP_HDR);
  if TCPDataLen > CheckDataLen then
    TCPDataLen := CheckDataLen;

  Common.AddPKTHdr(Item, Node);
  Common.AddETH(Item, ETHHdr);
  Common.AddIP(Item, IPHdr);
  Common.AddTCP(Item, IPHdr, TCPHdr);
  Common.AddTCPData(Item, TCPData, TCPDataLen, Global.TCPDataOffset);
end;

procedure TMainForm.lvUDPData1(Sender: TObject; Item: TListItem);
var
  i: Integer;
  Node: PSnoopMemoryNode;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  UDPHdr: PUDP_HDR;
  UDPData: PChar;
  UDPDataLen: Integer;
  CheckDataLen: Integer;
begin
  i := Item.Index;
  if i >= UDPList.Count then exit;
  Node := PSnoopMemoryNode(UDPList.Items[i]);
  ETHHdr := @Node^.Data[0];

  if not snoopIsIP(ETHHdr, @IPHdr) then exit;
  if not snoopIsUDP(IPHdr, @UDPHdr) then exit;
  snoopIsUDPData(IPHdr, UDPHdr, @UDPData, @UDPDataLen);
  CheckDataLen := ntohs(IPHdr^.Len) - sizeof(IP_HDR) - sizeof(UDP_HDR);
  if UDPDataLen > CheckDataLen then
    UDPDataLen := CheckDataLen;

  Common.AddPKTHdr(Item, Node);
  Common.AddETH(Item, ETHHdr);
  Common.AddIP(Item, IPHdr);
  Common.AddUDP(Item, IPHdr, UDPHdr);
  Common.AddUDPData(Item, UDPData, UDPDataLen, Global.UDPDataOffset);
end;

procedure TMainForm.lvTCPDataData(Sender: TObject; Item: TListItem);
var
  i: Integer;
  Node: PSnoopMemoryNode;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  TCPHdr: PTCP_HDR;
  TCPData: PChar;
  TCPDataLen: Integer;
  CheckDataLen: Integer;
begin
  i := Item.Index;
  if i >= TCPDataList.Count then exit;
  Node := PSnoopMemoryNode(TCPDataList.Items[i]);
  ETHHdr := @Node^.Data[0];

  if not snoopIsIP(ETHHdr, @IPHdr) then exit;
  if not snoopIsTCP(IPHdr, @TCPHdr) then exit;
  if not snoopIsTCPData(IPHdr, TCPHdr, @TCPData, @TCPDataLen) then exit;
  CheckDataLen := ntohs(IPHdr^.Len) - sizeof(IP_HDR) - sizeof(TCP_HDR);
  if TCPDataLen > CheckDataLen then
    TCPDataLen := CheckDataLen;

  Common.AddPKTHdr(Item, Node);
  Common.AddETH(Item, ETHHdr);
  Common.AddIP(Item, IPHdr);
  Common.AddTCP(Item, IPHdr, TCPHdr);
  Common.AddTCPData(Item, TCPData, TCPDataLen, Global.TCPDataOffset);
end;

procedure TMainForm.lvUDPDataData(Sender: TObject; Item: TListItem);
var
  i: Integer;
  Node: PSnoopMemoryNode;
  ETHHdr: PETH_HDR;
  IPHdr: PIP_HDR;
  UDPHdr: PUDP_HDR;
  UDPData: PChar;
  UDPDataLen: Integer;
  CheckDataLen: Integer;
begin
  i := Item.Index;
  if i >= UDPDataList.Count then exit;
  Node := PSnoopMemoryNode(UDPDataList.Items[i]);
  ETHHdr := @Node^.Data[0];

  if not snoopIsIP(ETHHdr, @IPHdr) then exit;
  if not snoopIsUDP(IPHdr, @UDPHdr) then exit;
  if not snoopIsUDPData(IPHdr, UDPHdr, @UDPData, @UDPDataLen) then exit;
  CheckDataLen := ntohs(IPHdr^.Len) - sizeof(IP_HDR) - sizeof(UDP_HDR);
  if UDPDataLen > CheckDataLen then
    UDPDataLen := CheckDataLen;

  Common.AddPKTHdr(Item, Node);
  Common.AddETH(Item, ETHHdr);
  Common.AddIP(Item, IPHdr);
  Common.AddUDP(Item, IPHdr, UDPHdr);
  Common.AddUDPData(Item, UDPData, UDPDataLen, Global.UDPDataOffset);
end;

end.

