unit Globals;

interface

uses
  SysUtils, Forms, Classes, IniFiles, Snoop;
  
type
  TGlobal = class
    IniFileName: String;
    // Coordination
    WindowState: Integer;
    Left: Integer;
    Top: Integer;
    Width: Integer;
    Height: Integer;
    ListViewHeight: Integer;
    ActiveListViewIndex: Integer;
    // OpenType
    OpenType: Integer;
    // Adapter Capture
    AdapterIndex: Integer;
    // Remote Capture
    Host: String;
    UserName: String;
    Password: String;
    DontShowJustOne: Boolean;
    // File Capture
    FileName: String;
    // Snoop Property
    Filter: String;
    Flags: Integer;
    ReadTimeout: Integer;
    SnapLen: Integer;
    ThreadSafe: Boolean;
    // ViewHexa;
    ViewHexa: Boolean;
    // FilterList
    FilterList: TStringList;
    // Option
    TCPDataOffset: Integer;
    UDPDataOffset: Integer;

    constructor Create;
    destructor Destroy; override;
    procedure Load;
    procedure Save;
  end;

var
  Global: TGlobal;

implementation

constructor TGlobal.Create;
begin
  inherited;
  FilterList := TStringList.Create;
  IniFileName := ExtractFilePath(Application.ExeName) + 'Expert.ini';
  Load;
end;

destructor TGlobal.Destroy;
begin
  Save;
  if FilterList <> nil then
  begin
    FilterList.Free;
    FilterList := nil;
  end;
  inherited;
end;

procedure TGlobal.Load;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(IniFileName);
  // Coordination
  WindowState := IniFile.ReadInteger('Coordination', 'WindowState', Integer(wsNormal));
  if WindowState = Integer(wsMaximized) then
  begin
    Left := 50;
    Top := 50;
    Width := Screen.Width - 100;
    Height := Screen.Height - 100;
  end else
  begin
    Left := IniFile.ReadInteger('Coordination', 'Left', 50);
    Top := IniFile.ReadInteger('Coordination', 'Top', 50);
    Width := IniFile.ReadInteger('Coordination', 'Width', Screen.Width - 100);
    Height := IniFile.ReadInteger('Coordination', 'Height', Screen.Height - 100);
  end;
  ListViewHeight := IniFile.ReadInteger('Coordination', 'ListViewHeight', 300);
  ActiveListViewIndex := IniFile.ReadInteger('Coordination', 'ActiveListViewIndex', 0);
  // Open Type
  OpenType := IniFile.ReadInteger('Option', 'OpenType', 0);
  // Adapter Capture
  AdapterIndex := IniFile.ReadInteger('AdapterCapture', 'AdapterIndex', -1);
  // Remote Capture
  Host := IniFile.ReadString('RemoteCapture', 'Host', '');
  UserName := IniFile.ReadString('RemoteCapture', 'UserName', '');
  Password := IniFile.ReadString('RemoteCapture', 'Password', '');
  DontShowJustOne := IniFile.ReadBool('RemoteCapture', 'DontShowJustOne', false);
  // File Capture
  FileName := IniFile.ReadString('FileCapture', 'FileName', '');
  // Snoop Property
  Filter := IniFile.ReadString('Snoop', 'Filter', '');
  Flags := IniFile.ReadInteger('Snoop', 'Flags', 1);
  ReadTimeout := IniFile.ReadInteger('Snoop', 'ReadTimeout', 1);
  SnapLen := IniFile.ReadInteger('Snoop', 'SnapLen', SNOOP_DEFAULT_SNAPLEN);
  ThreadSafe := IniFile.ReadBool('Snoop', 'ThreadSafe', true);
  // ViewHexa;
  ViewHexa := IniFile.ReadBool('Option', 'ViewHexa', false);
  // FilterList
  FilterList.CommaText := IniFile.ReadString('Option', 'FilterList', '');
  // Option
  TCPDataOffset := IniFile.ReadInteger('Option', 'TCPDataOffset', 0);
  UDPDataOffset := IniFile.ReadInteger('Option', 'UDPDataOffset', 0);

  IniFile.Free;
end;

procedure TGlobal.Save;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(IniFileName);
  try
    // Coordination
    IniFile.WriteInteger('Coordination', 'WindowState', WindowState);
    IniFile.WriteInteger('Coordination', 'Left', Left);
    IniFile.WriteInteger('Coordination', 'Top', Top);
    IniFile.WriteInteger('Coordination', 'Width', Width);
    IniFile.WriteInteger('Coordination', 'Height', Height);
    IniFile.WriteInteger('Coordination', 'ListViewHeight', ListViewHeight);
    IniFile.WriteInteger('Coordination', 'ActiveListViewIndex', ActiveListViewIndex);
    // Open Type
    IniFile.WriteInteger('Option', 'OpenType', OpenType);
    // Adapter Capture
    IniFile.WriteInteger('AdapterCapture', 'AdapterIndex', AdapterIndex);
    // Remote Capture
    IniFile.WriteString('RemoteCapture', 'Host', Host);
    IniFile.WriteString('RemoteCapture', 'UserName', UserName);
    IniFile.WriteString('RemoteCapture', 'Password', Password);
    IniFile.WriteBool('RemoteCapture', 'DontShowJustOne', DontShowJustOne);
    // File Capture
    IniFile.WriteString('FileCapture', 'FileName', FileName);
    // Snoop Property
    IniFile.WriteString('Snoop', 'Filter', Filter);
    IniFile.WriteInteger('Snoop', 'Flags', Flags);
    IniFile.WriteInteger('Snoop', 'ReadTimeout', ReadTimeout);
    IniFile.WriteInteger('Snoop', 'SnapLen', SnapLen);
    IniFile.WriteBool('Snoop', 'ThreadSafe', ThreadSafe);
    // ViewHexa
    IniFile.WriteBool('Option', 'ViewHexa', ViewHexa);
    // FilterList
    IniFile.WriteString('Option', 'FilterList', FilterList.CommaText);
    // Option
    IniFile.WriteInteger('Option', 'TCPDataOffset', TCPDataOffset);
    IniFile.WriteInteger('Option', 'UDPDataOffset', UDPDataOffset);

  finally
    IniFile.Free;
  end;
end;

end.
