unit Globals;

interface

uses
  SysUtils, Forms, IniFiles;

type
  TGlobal = class
  public
    // Coordination
    MainLeft: Integer;
    MainTop: Integer;
    MainWidth: Integer;
    MainHeight: Integer;
    // WinPcap
    AdapterIndex: Integer;
    Filter: String;
    Flags: Integer;
    ReadTimeOut: Integer;
    SnapLen: Integer;
    ThreadSafe: Boolean;

    constructor Create;
    destructor Destroy; override;
    procedure Load;
    procedure Save;
  end;

var
  Global: TGlobal;

implementation

constructor TGlobal.Create;
begin
  inherited;
  Load;
end;

destructor TGlobal.Destroy;
begin
  Save;
  inherited;
end;

procedure TGlobal.Load;
var
  Path: String;
  IniFile: TIniFile;
begin
  Path := ExtractFilePath(Application.ExeName);
  IniFile := TIniFile.Create(Path + 'TCPBlock.ini');
  // Coordination
  MainLeft := IniFile.ReadInteger('Coordination', 'MainLeft', 0);
  MainTop := IniFile.ReadInteger('Coordination', 'MainTop', 0);
  MainWidth := IniFile.ReadInteger('Coordination', 'MainWidth', 640);
  MainHeight := IniFile.ReadInteger('Coordination', 'MainHeight', 480);
  // WinPcap
  AdapterIndex := IniFile.ReadInteger('WinPcap', 'AdapterIndex', -1);
  Filter := IniFile.ReadString('WinPcap', 'Filter', '');
  Flags := IniFile.ReadInteger('WinPcap', 'Flags', 1);
  ReadTimeOut := IniFile.ReadInteger('WinPcap', 'ReadTimeOut', 1);
  SnapLen := IniFile.ReadInteger('WinPcap', 'SnapLen', 54); // ETH_HDR + IP_HDR _ TCP_HDR

  IniFile.Free;
end;

procedure TGlobal.Save;
var
  Path: String;
  IniFile: TIniFile;
begin
  Path := ExtractFilePath(Application.ExeName);
  IniFile := TIniFile.Create(Path + 'TCPBlock.ini');
  try
    // Coordination
    IniFile.WriteInteger('Coordination', 'MainLeft', MainLeft);
    IniFile.WriteInteger('Coordination', 'MainTop', MainTop);
    IniFile.WriteInteger('Coordination', 'MainWidth', MainWidth);
    IniFile.WriteInteger('Coordination', 'MainHeight', MainHeight);
    // WinPcap
    IniFile.WriteInteger('WinPcap', 'AdapterIndex', AdapterIndex);
    IniFile.WriteString('WinPcap', 'Filter', Filter);
    IniFile.WriteInteger('WinPcap', 'Flags', Flags);
    IniFile.WriteInteger('WinPcap', 'ReadTimeOut', ReadTimeOut);
    IniFile.WriteInteger('WinPcap', 'SnapLen', SnapLen);
  finally
    IniFile.Free;
  end;
end;


end.
