// ----------------------------------------------------------------------------
//
// Snoop Component Suite version 3.0
//
// http://www.netlab.co.kr
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------
	
#ifndef __SNOOP_H__
#define __SNOOP_H__

#ifdef _DEBUG
#pragma comment(lib, "SnoopD.lib") 
#else
#pragma comment(lib, "Snoop.lib")
#endif
#pragma comment(lib, "Iphlpapi.lib")
#pragma comment(lib, "Packet.lib")
#pragma comment(lib, "wpcap.lib")

#include "winsock2.h"
#define  _WINSOCKAPI_ // for prevent include "winsock.h"
#include "Iphlpapi.h"
//#include "Iptypes.h" // gilgil temp 2006.10.24

#include "VDDelphi.h"
#include "VDTrace.h"
#include "VDThread.h"

#define WPCAP
#define HAVE_REMOTE
#include "pcap.h"
//#include "remote-ext.h"
// #include "MACADDR.h" // gilgil temp 2006.10.25

// ----------------------------------------------------------------------------
// Constant
// ----------------------------------------------------------------------------
#define SNOOP_DEFAULT_READTIMEOUT      1
#define SNOOP_DEFAULT_SNAPLEN          1600
#define SNOOP_MAC_SIZE                 6

typedef pcap_pkthdr                    PCAP_PKTHDR;

// ----------------------------------------------------------------------------
// Type Definition
// ----------------------------------------------------------------------------

typedef struct _MACADDR
{
	UINT8 s[SNOOP_MAC_SIZE];
} MACADDR;

// ----------------------------------------------------------------------------
// ETH Hdr
// ----------------------------------------------------------------------------
#define PROTO_IP                       0x0800
#define PROTO_ARP                      0x0806
#define PROTO_XNS                      0x0600
#define PROTO_SNMP                     0x814C
#define PROTO_OLD_IPX                  0x8137
#define PROTO_NOVELL                   0x8138
#define PROTO_IPNG                     0x86DD

typedef struct _ETHERNET_HDR
{
    MACADDR Destination;               // destination Mac Address
    MACADDR Source;                    // source Mac Address
    UINT16 Protocol;                   // protocol
} ETH_HDR;

// ----------------------------------------------------------------------------
// IP Hdr(IPv4)
// ----------------------------------------------------------------------------
#define PROTO_ICMP                     0x01
#define PROTO_TCP                      0x06
#define PROTO_UDP                      0x11

typedef struct _IP_HDR
{
    UINT8  VerLen;                     // high 4bit header length, low 4bit version
    UINT8  Service;                    // type of service
    UINT16 Len;                        // total length 
    UINT16 Ident;                      // identification 
    UINT16 FlagOff;                    // fragment offset field
    UINT8  TimeLive;                   // time to live
    UINT8  Protocol;                   // protocol
    UINT16 Checksum;                   // checksum
    UINT32 Source;                     // source ip address
    UINT32 Destination;                // destination ip address
} IP_HDR;

// ----------------------------------------------------------------------------
// ARP Hdr
// ----------------------------------------------------------------------------
#define ARP_REQUEST                    0x0001
#define ARP_REPLY                      0x0002

typedef struct _ARP_HDR
{
   UINT16  HardwareType;
   UINT16  ProtocolType;
   UINT8   HLen;
   UINT8   PLen;
   UINT16  Operation;
   MACADDR SenderHA;
   UINT32  SenderIP;
   MACADDR TargetHA;
   UINT32  TargetIP;
} ARP_HDR;

// ----------------------------------------------------------------------------
// ICMP Hdr
// ----------------------------------------------------------------------------
typedef struct _ICMP_HDR
{
   UINT8   Type;
   UINT8   Code;
   UINT16  Checksum;
   char   *Data;
} ICMP_HDR;

// ----------------------------------------------------------------------------
// TCP Hdr
// ----------------------------------------------------------------------------
#define TCP_FLAG_FIN                   0x01
#define TCP_FLAG_SYN                   0x02
#define TCP_FLAG_RST                   0x04
#define TCP_FLAG_PSH                   0x08
#define TCP_FLAG_ACK                   0x10
#define TCP_FLAG_URG                   0x20

typedef struct _TCP_HDR
{
   UINT16  Source;
   UINT16  Destination;
   UINT32  Seq;
   UINT32  Ack;
   UINT8   Off_Rsvd;
   UINT8   Rsvd_Flags;
   UINT16  Window;
   UINT16  Checksum;
   UINT16  UrgPoint;
} TCP_HDR;

// ----------------------------------------------------------------------------
// UDP Hdr
// ----------------------------------------------------------------------------
typedef struct _UDP_HDR
{
   UINT16  Source;
   UINT16  Destination;
   UINT16  Len;
   UINT16  Checksum;
} UDP_HDR;

// ----------------------------------------------------------------------------
// CSnoopThread
// ----------------------------------------------------------------------------
class CCustomSnoop;
class CSnoopThread : public CVDThread
{
public:
   CCustomSnoop *m_pSnoop;
public:
    CSnoopThread(CCustomSnoop *pSnoop);
   virtual ~CSnoopThread();
   void Open();
   void Close();
   virtual void Run();
};

// ----------------------------------------------------------------------------
// CCustomSnoop
// ----------------------------------------------------------------------------
class CCustomSnoop
{
public:
    // property
    bool   m_bActive;
    int    m_nAdapterIndex;
    string m_strAdapterName;
    string m_strAdapterDescription;
    CVDStringList m_strAdapterNames;
    CVDStringList m_strAdapterDescriptions;
    bool   m_bCaptureEnabled;
    bool   m_bThreadSafe;
    string m_strFilter;
    int    m_nSnapLen;
    int    m_nFlags;
    int    m_nReadTimeout;
    string m_strError;
    string m_strSourceName;

    virtual void OnGetRemoteAdapterInfo(void *pSender,
		CVDStringList *pAdapterNames,
		CVDStringList *pAdapterDescriptions,
		int &nAdapterIndex);
    void SetAdapterIndex(int nValue);
    void SetAdapterName(string strValue);
    void SetAdapterDescription(string strValue);
    void DoOpen(char *pSource, pcap_rmtauth *pAuth, pcap_if_t *pDev);
    void CreateSnoopThread();
public:
    pcap *m_pPcap;
protected:
    // used internally
    CSnoopThread *m_pThread;
    PCAP_PKTHDR *m_pPKTHdr;
    ETH_HDR *m_pPKTData;
    virtual void OnCaptureCB()/* = 0*/; // gilgil temp 2006.10.03
    void GetAdapterNames();
    void GetAdapterDescriptions();
    bool ProcessFilter(pcap_if_t *pDev);
public:
    CCustomSnoop();
    virtual ~CCustomSnoop();
    void Open();
    void Open(string strHost, string strUserName, string strPassword);
    bool LoadFromFile(string strFileName);
    void Close();
protected:
	friend class CSnoopThread;
};

// ----------------------------------------------------------------------------
// CSnoop
// ----------------------------------------------------------------------------
class CSnoop : public CCustomSnoop
{
protected:
    virtual void OnCaptureCB(); // override;
public:
	CSnoop() {}
	virtual ~CSnoop() {};
public:
    virtual void OnCapture(
		void        *pSender,
		PCAP_PKTHDR *pPKTHdr,
		ETH_HDR     *pETHHdr);
};

// ----------------------------------------------------------------------------
// CSnoopIP
// ----------------------------------------------------------------------------
class CSnoopIP : public CCustomSnoop
{
protected:
	virtual void OnCaptureCB(); // override;
public:
	CSnoopIP() { m_strFilter = "ip"; }
	virtual ~CSnoopIP() {}
public:
	virtual void OnCaptureIP(
		void        *pSender,
		PCAP_PKTHDR *pPKTHdr,
		ETH_HDR     *pETHHdr,
		IP_HDR      *pIPHdr);
};

// ----------------------------------------------------------------------------
// CSnoopARP
// ----------------------------------------------------------------------------
class CSnoopARP : public CCustomSnoop
{
protected:
	virtual void OnCaptureCB(); // override;
public:
	CSnoopARP() { m_strFilter = "arp"; }
	virtual ~CSnoopARP() {}
public:
	virtual void OnCaptureARP(
		void        *pSender,
		PCAP_PKTHDR *pPKTHdr,
		ETH_HDR     *pETHHdr,
		ARP_HDR     *pARPHdr);
};

// ----------------------------------------------------------------------------
// CSnoopICMP
// ----------------------------------------------------------------------------
class CSnoopICMP : public CCustomSnoop
{
protected:
	virtual void OnCaptureCB(); // override;
public:
	CSnoopICMP() { m_strFilter = "icmp"; }
	virtual ~CSnoopICMP() {}
public:
	virtual void OnCaptureICMP(
		void        *pSender,
        PCAP_PKTHDR *pPKTHdr,
        ETH_HDR     *pETHHdr,
		IP_HDR      *pIPHdr,
		ICMP_HDR    *pICMPHdr);
};

// ----------------------------------------------------------------------------
// CSnoopTCP
// ----------------------------------------------------------------------------
class CSnoopTCP : public CCustomSnoop
{
protected:
	virtual void OnCaptureCB(); // override;
public:
	CSnoopTCP() { m_strFilter = "tcp"; }
	virtual ~CSnoopTCP() {}
public:
	virtual void OnCaptureTCP(
		void        *pSender,
		PCAP_PKTHDR *pPKTHdr,
		ETH_HDR     *pETHHdr,
		IP_HDR      *pIPHdr,
		TCP_HDR     *pTCPHdr);
};

// ----------------------------------------------------------------------------
// CSnoopTCPData
// ----------------------------------------------------------------------------
class CSnoopTCPData : public CCustomSnoop
{
protected:
	virtual void OnCaptureCB(); // override;
public:
	CSnoopTCPData() { m_strFilter = "tcp"; }
	virtual ~CSnoopTCPData() {}
public:
	virtual void OnCaptureTCPData(
		void        *pSender,
		PCAP_PKTHDR *pPKTHdr,
		ETH_HDR     *pETHHdr,
		IP_HDR      *pIPHdr,
		TCP_HDR     *pTCPHdr,
		char        *pTCPData,
		int          nTCPDataLen);
};

// ----------------------------------------------------------------------------
// CSnoopUDP
// ----------------------------------------------------------------------------
class CSnoopUDP : public CCustomSnoop
{
protected:
	virtual void OnCaptureCB(); // override;
public:
	CSnoopUDP() { m_strFilter = "udp"; }
	virtual ~CSnoopUDP() {}
public:
	virtual void OnCaptureUDP(
		void        *pSender,
		PCAP_PKTHDR *pPKTHdr,
		ETH_HDR     *pETHHdr,
		IP_HDR      *pIPHdr,
		UDP_HDR     *pUDPHdr);
};

// ----------------------------------------------------------------------------
// CSnoopUDPData
// ----------------------------------------------------------------------------
class CSnoopUDPData : public CCustomSnoop
{
protected:
	virtual void OnCaptureCB(); // override;
public:
	CSnoopUDPData() { m_strFilter = "udp"; }
	virtual ~CSnoopUDPData() {}
public:
	virtual void OnCaptureUDPData(
		void        *pSender,
		PCAP_PKTHDR *pPKTHdr,
		ETH_HDR     *pETHHdr,
		IP_HDR      *pIPHdr,
		UDP_HDR     *pUDPHdr,
		char        *pUDPData,
		int          nUDPDataLen);
};

// ----------------------------------------------------------------------------
// CSnoopStatistics
// ----------------------------------------------------------------------------
class CSnoopStatistics : public CCustomSnoop
{
protected:
	virtual void OnCaptureCB(); // override;
public:
	CSnoopStatistics();
	virtual ~CSnoopStatistics();
	void Open();
public:
	virtual void OnCaptureStatistics(
		void         *pSender,
		PCAP_PKTHDR  *pPKTHdr,
		__int64       nPackets,
		__int64       nBytes);
};

// ----------------------------------------------------------------------------
// CSnoopDump
// ----------------------------------------------------------------------------
class CSnoopDump
{
	// not supported yet
};

// ----------------------------------------------------------------------------
// CSnoopMyNetwork
// ----------------------------------------------------------------------------
class CSnoopMyNetwork 
{
protected:
	CSnoop *m_pSnoop;
public:
	UINT32 m_uIP;
	MACADDR m_sMac;
	UINT32 m_uSubnetMask;
	UINT32 m_uGateway;
public:
	CSnoopMyNetwork();
	virtual ~CSnoopMyNetwork();
	void Refresh();
public:
	int    GetAdapterIndex();
	void   SetAdapterIndex(int nValue);
	string GetAdapterName();
	string GetAdapterDescription();
	void   SetAdapterName(string strValue);
	void   SetAdapterDescription(string strValue);
	CVDStringList* GetAdapterNames();
	CVDStringList* GetAdapterDescriptions();
	string GetStrIP();
	string GetStrMac();
	string GetStrGateway();
	string GetStrSubnetMask();
	void   SetStrIP(string strValue);
	void   SetStrMac(string strValue);
	void   SetStrGateway(string strValue);
	void   SetStrSubnetMask(string strValue);
};

// ----------------------------------------------------------------------------
// General Functions
// ----------------------------------------------------------------------------
bool    snoopIsIP(ETH_HDR *pETHHdr, IP_HDR **ppIPHdr = NULL);
bool    snoopIsARP(ETH_HDR *pETHHdr, ARP_HDR **ppARPHdr = NULL);
bool    snoopIsICMP(IP_HDR *pIPHdr, ICMP_HDR **ppICMPHdr = NULL);
bool    snoopIsTCP(IP_HDR *pIPHdr, TCP_HDR **ppTCPHdr = NULL);
bool    snoopIsUDP(IP_HDR *pIPHdr, UDP_HDR **ppUDPHdr = NULL);
bool    snoopIsTCPData(IP_HDR *pIPHdr, TCP_HDR *pTCPHdr, char **ppTCPData = NULL, int *pTCPDataLen = NULL);
bool    snoopIsUDPData(IP_HDR *pIPHdr, UDP_HDR *pUDPHdr, char **ppUDPData = NULL, int *pUDPDataLen = NULL);

string  snoopMac2Str(MACADDR sMac);
string  snoopMac2StrFormat(MACADDR sMac, string strFormat);
string  snoopIP2Str(UINT32 IP);
MACADDR snoopStr2Mac(string s);
UINT32   snoopStr2IP(string s);
MACADDR snoopCleanMac();
bool    snoopIsCleanMac(MACADDR *pMac);
int     snoopCompareMac(MACADDR *pMac1, MACADDR *pMac2);

UINT16  snoopIPChecksum(IP_HDR *pIPHdr);
UINT16  snoopTCPChecksum(IP_HDR *pIPHdr, TCP_HDR *pTCPHdr);
UINT16  snoopUDPChecksum(IP_HDR *pIPHdr, UDP_HDR *pUDPHdr);
UINT16  snoopRecalculateChecksum(UINT16 OldChecksum, UINT16 OldValue, UINT16 NewValue);
int     snoopSendPacket(pcap *pcap, char *pBuffer, int nSize);

#endif  __SNOOP_H__