// ----------------------------------------------------------------------------
//
// Snoop Component Suite version 3.0
//
// http://www.netlab.co.kr
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------

#ifndef __SNOOPSPECIAL_H__
#define __SNOOPSPECIAL_H__

#include "Snoop.h"
#include "VDSyncObjs.h"

// ----------------------------------------------------------------------------
// Constant
// ----------------------------------------------------------------------------
#define ARP_TABLE_COUNT                1600
#define ARP_SPOOF_TABLE_COUNT          512
#define MAX_RELAY_SIZE                 2000

// ----------------------------------------------------------------------------
// CSnoopFindMac
// ----------------------------------------------------------------------------
class CSnoopFindMac : public CCustomSnoop
{
protected:
	// used internally
	CSnoopMyNetwork *m_pSnoopMyNetwork;
	virtual void OnCaptureCB(); // override;
public:
	CSnoopFindMac();
	virtual ~CSnoopFindMac();
	void Open();
	void SendRequest(UINT32 uIP);
public:
	virtual void OnCaptureFindMac(
		void *pSender, UINT32 uIP, MACADDR *pMac)/* = 0*/;
};

// ----------------------------------------------------------------------------
// CARPTable
// ----------------------------------------------------------------------------
class CARPTable
{
	bool    m_bActive[ARP_TABLE_COUNT];
	UINT32   m_uIP[ARP_TABLE_COUNT];
	MACADDR m_sMac[ARP_TABLE_COUNT];
public:
	CARPTable();
	virtual ~CARPTable();
public:
	void Clear();
	int Add(UINT32 uIP, MACADDR *pMac);
	int Find(UINT32 uIP);
};

// ----------------------------------------------------------------------------
// CARPSpoofTable
// ----------------------------------------------------------------------------
class CARPSpoofTable
{
public:
	bool    m_bActive[ARP_SPOOF_TABLE_COUNT];
	UINT32   m_uSenderIP[ARP_SPOOF_TABLE_COUNT];
	MACADDR m_sSenderMac[ARP_SPOOF_TABLE_COUNT];
	UINT32   m_uTargetIP[ARP_SPOOF_TABLE_COUNT];
	MACADDR m_sTargetMac[ARP_SPOOF_TABLE_COUNT];
public:
	CARPSpoofTable();
	virtual ~CARPSpoofTable();
public:
	void Clear();
	int Add(UINT32 uSenderIP, UINT32 uTargetIP);
	int Update(UINT32 uSenderIP, MACADDR *pSenderMac, UINT32 uTargetIP, MACADDR *pTargetMac);
	int Delete(int nIndex);
	int Delete(UINT32 uSenderIP, UINT32 uTargetIP);
	int Delete(UINT32 uSenderIP, MACADDR *pSenderMac, UINT32 uTargetIP, MACADDR *pTargetMac);
	int Find(UINT32 uIP);
	int Find(UINT32 uSenderIP, UINT32 uTargetIP); 
	int Find(UINT32 uSenderIP, MACADDR *pSenderMac, UINT32 uTargetIP, MACADDR *pTargetMac);
	int Count();
};

// ----------------------------------------------------------------------------
// CSnoopARPSpoofThread
// ----------------------------------------------------------------------------
class CSnoopARPSpoof;
class CSnoopARPSpoofThread : public CVDThread
{
protected:
	CSnoopARPSpoof *m_pOwner;
protected:
	CVDEvent *m_pThreadEvent;
public:
	CSnoopARPSpoofThread(CSnoopARPSpoof *pOwner);
	virtual ~CSnoopARPSpoofThread();
public:
	void Open();
	void Close();
	virtual void Run(); // override
};

// ----------------------------------------------------------------------------
// CSnoopARPSpoof
// ----------------------------------------------------------------------------
class CMySnoopFindMac : public CSnoopFindMac
{
public:
	virtual void OnCaptureFindMac(
		void *pSender, UINT32 uIP, MACADDR *pMac); // override;
};

class CMySnoopARP : public CSnoopARP
{
public:
	virtual void OnCaptureARP(
		void         *pSender,
		PCAP_PKTHDR  *pPKTHdr,
		ETH_HDR *pETHHdr,
		ARP_HDR      *pARPHdr); // override;
};

class CSnoopARPSpoof : public CCustomSnoop
{
protected:
	// used internally
	CVDEvent *m_pSnoopFindMacEvent;
	CMySnoopFindMac *m_pSnoopFindMac;
	CMySnoopARP *m_pSnoopARP;
protected:
	// used internally
	CSnoopARPSpoofThread *m_pSnoopARPSpoofThread;
	bool SetARPSpoofTableMac();
	void SendARPSpoof(int nIndex);
	void SendARPSpoofAll();
	void SendARPRecover(int nIndex);
	void SendARPRecoverAll();
	void PreventARPRecover(ETH_HDR *pEhternetHeader, ARP_HDR *pARPHdr);
protected:
	bool m_bClosing; // used in Close method
protected:
	CARPTable *m_pARPTable;
public:
	CARPSpoofTable *m_pARPSpoofTable;
	int m_nRefreshTime;
protected:
	virtual void OnCaptureCB(); // override;
public:
	CSnoopARPSpoof();
	virtual ~CSnoopARPSpoof();
public:
	void Open();
	void Close();
public:
	virtual OnCaptureARPSpoofEvent(
		void         *pSender, 
		PCAP_PKTHDR  *pPKTHdr,
		ETH_HDR *pETHHdr,
		IP_HDR       *pIPHdr,
		bool *pRelay)/* = 0*/;
public:
	MACADDR m_sVirtualMac;
};

// ----------------------------------------------------------------------------
// General Function
// ----------------------------------------------------------------------------
void snoopSendARP(pcap_t *ppcap,
	ETH_HDR *pETHHdr,
	ARP_HDR *pARPHdr);
void snoopSendTCPBlock(pcap_t *ppcap,
	MACADDR *pMyMac,
	ETH_HDR *pETHHdr,
	IP_HDR *pIPHdr,
	TCP_HDR *pTCPHeader);
void snoopSendHTTPBlock(pcap_t *ppcap,
	MACADDR *pMyMac,
	ETH_HDR *pETHHdr,
	IP_HDR *pIPHdr,
	TCP_HDR *pTCPHeader,
	char* pTCPData,
	int nTCPDataLength,
	char* pBlockHTML,
	int nBlockHTMLLength);

#endif  __SNOOPSPECIAL_H__