#include "Snoop.h"

#pragma comment(lib, "Ws2_32.lib")
#pragma comment(lib, "wpcap.lib")

// ----------------------------------------------------------------------------
// CSnoopThread
// ----------------------------------------------------------------------------
CSnoopThread::CSnoopThread(CCustomSnoop *pSnoop)
{
	m_pSnoop = pSnoop;
};

CSnoopThread::~CSnoopThread()
{
}

void CSnoopThread::Open()
{
	CVDThread::Open();
}

void CSnoopThread::Close()
{
	CVDThread::Close();
	VDWaitFor();
}

void CSnoopThread::Run()
{
	int i;
	CCustomSnoop *pSnoop;
	pcap *pcap;
	bool bThreadSafe;
	PCAP_PKTHDR *pPKTHdr;
	ETH_HDR *pPKTData;

	pSnoop = m_pSnoop; // use local variable for execute optimization
	if (pSnoop == NULL)
		return;
	pcap = pSnoop->m_pPcap;
	bThreadSafe = pSnoop->m_bThreadSafe;
	while (!m_bTerminated)
	{
		i = pcap_next_ex(pcap, &pPKTHdr, (const u_char**)&pPKTData);
		if (m_bTerminated) break;
		switch (i) {
		case -1: // if (an error occurred
			pSnoop->m_strError = "Snoop Error: pcap_next_ex return -1";
			break;
		case 0:
			continue; // if (the timeout set with pcap_open_live() has elapsed.
		}
		pSnoop->m_pPKTHdr = pPKTHdr;
		pSnoop->m_pPKTData = pPKTData;
		// ----- gilgil temp 2006.10.02 -----
		/*
		if (bThreadSafe)
			VDSynchronize(pSnoop->OnCaptureCB());
		else
			pSnoop->OnCaptureCB();
		*/
		pSnoop->OnCaptureCB();
		// ----------------------------------
	}
}

// ----------------------------------------------------------------------------
// CCustomSnoop
// ----------------------------------------------------------------------------
void CCustomSnoop::OnGetRemoteAdapterInfo(void *pSender,
	CVDStringList *pAdapterNames,
	CVDStringList *pAdapterDescriptions,
	int &nAdapterIndex)
{
	// you must implenent this method.
}

void CCustomSnoop::SetAdapterIndex(int nValue)
{
	if (nValue == m_nAdapterIndex)
		return;
	if ((nValue < 0) || (nValue >= m_strAdapterNames.m_nCount))
	{
		m_nAdapterIndex = -1;
		m_strAdapterName = "";
	} else {
		m_nAdapterIndex = nValue;
		m_strAdapterName = m_strAdapterNames.GetAt(nValue);
		m_strAdapterDescription = m_strAdapterDescriptions.GetAt(nValue);
	}
}

void CCustomSnoop::SetAdapterName(string strValue)
{
    int i;

    if (strValue == m_strAdapterName)
        return;
    for (i = 0; i < m_strAdapterNames.m_nCount; i++)
    {
        if (strValue == m_strAdapterNames.GetAt(i))
        {
            m_nAdapterIndex = i;
            m_strAdapterName = m_strAdapterNames.GetAt(i);
            m_strAdapterDescription = m_strAdapterDescriptions.GetAt(i);
            return;
        }
    }
    m_nAdapterIndex = -1;
    m_strAdapterName = "";
	m_strAdapterDescription = "";
}

void CCustomSnoop::SetAdapterDescription(string strValue)
{
    int i;

    if (strValue == m_strAdapterDescription)
        return;
    for (i = 0; i < m_strAdapterDescriptions.m_nCount; i++)
    {
        if (strValue == m_strAdapterDescriptions.GetAt(i))
        {
            m_nAdapterIndex = i;
            m_strAdapterName = m_strAdapterNames.GetAt(i);
            m_strAdapterDescription = m_strAdapterDescriptions.GetAt(i);
            return;
        }
    }
    m_nAdapterIndex = -1;
    m_strAdapterName = "";
    m_strAdapterDescription = "";
}

void CCustomSnoop::DoOpen(char *pSource, pcap_rmtauth *pAuth, pcap_if_t *pDev)
{
    char szErrBuf[PCAP_ERRBUF_SIZE];
	int nDataLink;

    m_pPcap = pcap_open(pSource, m_nSnapLen, m_nFlags, m_nReadTimeout, pAuth, szErrBuf);
    if (m_pPcap == NULL)
    {
        m_strError = szErrBuf;
        return;
    }
	// ----- gilgil temp 2007.05.03 -----
	nDataLink = pcap_datalink(m_pPcap);
	if (nDataLink != DLT_EN10MB)
		GTRACE("[Snoop.cpp] CCustomSnoop::DoOpen pcap_datalink=%d", nDataLink);
	// ----------------------------------
    m_strSourceName = (string)pSource;
    if (!ProcessFilter(pDev)) return;
    m_bActive = true;
}

void CCustomSnoop::CreateSnoopThread()
{
    // Start Snoop Read Thread
    m_pThread = new CSnoopThread(this);
    m_pThread->m_bFreeOnTerminate = false;
    m_pThread->Open();
}

void CCustomSnoop::OnCaptureCB()
{
	// you must implement this method
	GTRACE("[Snoop.cpp] ***********************************************");
	GTRACE("[Snoop.cpp] CCustomSnoop::OnCaptureCB virtual function call");
	GTRACE("[Snoop.cpp] ***********************************************");
}

void CCustomSnoop::GetAdapterNames()
{
    int i;
    pcap_if_t *pDev;
    pcap_if_t *pAllDevs;
    char szErrBuf[PCAP_ERRBUF_SIZE];

    m_strAdapterNames.m_nCapacity = 10;
    i = pcap_findalldevs(&pAllDevs, szErrBuf);
    if (i != 0) // if (error occured
    {
        m_strError = szErrBuf;
        return;
    }
    pDev = pAllDevs;
    while (pDev != NULL)
    {
        m_strAdapterNames.Add((string)pDev->name);
        pDev = pDev->next;
    }
    m_strError = "";
    pcap_freealldevs(pAllDevs);
}

void CCustomSnoop::GetAdapterDescriptions()
{
    int i;
    pcap_if_t *pDev;
    pcap_if_t *pAllDevs;
	char szErrBuf[PCAP_ERRBUF_SIZE];

    m_strAdapterDescriptions.m_nCapacity = 10;
    i = pcap_findalldevs(&pAllDevs, szErrBuf);
    if (i != 0) // if (error occured
    {
        m_strError = szErrBuf;
        return;
    }
    pDev = pAllDevs;
    while (pDev != NULL)
    {
        m_strAdapterDescriptions.Add((string)pDev->description);
        pDev = pDev->next;
    }
    m_strError = "";
    pcap_freealldevs(pAllDevs);
}

bool CCustomSnoop::ProcessFilter(pcap_if_t *pDev)
{
    u_int uNetMask;
    bpf_program FCode;

    if ((pDev != NULL) && (pDev->addresses != NULL))
        uNetMask = ((struct sockaddr_in*)(pDev->addresses->netmask))->sin_addr.S_un.S_addr;
    else
        uNetMask = 0xFFFFFFFF;
    if (pcap_compile(m_pPcap, &FCode, (char*)m_strFilter, 1, uNetMask) < 0 )
    {
        m_strError = string(pcap_geterr(m_pPcap));
        return false;
    }
    if (pcap_setfilter(m_pPcap, &FCode) < 0)
    {
        m_strError = string(pcap_geterr(m_pPcap));
        return false;
    }
    return true;
}

CCustomSnoop::CCustomSnoop()
{
    m_bActive = false;
    m_nAdapterIndex = -1;
    m_strAdapterName = "";
    GetAdapterNames();
    GetAdapterDescriptions();
    m_bCaptureEnabled = true;
    m_bThreadSafe = true;
    m_strFilter = "";
    m_nSnapLen = SNOOP_DEFAULT_SNAPLEN;
    m_nFlags = PCAP_OPENFLAG_PROMISCUOUS;
    m_nReadTimeout = SNOOP_DEFAULT_READTIMEOUT;
    m_strError = "";
    m_pThread = NULL;
    m_pPcap = NULL;
}

CCustomSnoop::~CCustomSnoop()
{
    Close();
}

void CCustomSnoop::Open()
{
    int i;
    pcap_if_t *pDev;
    pcap_if_t *pAllDevs;
    char szErrBuf[PCAP_ERRBUF_SIZE];

    if (m_bActive)
    {
        m_strError = "Snoop Error: Already open";
        return;
    }
    i = pcap_findalldevs(&pAllDevs, szErrBuf);
    if (i != 0) // if (error occured)
    {
        m_strError = szErrBuf;
        return;
    }
    pDev = pAllDevs;
    while (pDev != NULL)
    {
		if (strcmp(pDev->name, (char*)m_strAdapterName) == 0) break;
        pDev = pDev->next;
    }
    if (pDev == NULL)
        m_strError = "Snoop Error: Invalid AdapterName";
    else
    {
        DoOpen((char*)m_strAdapterName, NULL, pDev);
        if (m_bActive && m_bCaptureEnabled)
            CreateSnoopThread();
    }
    pcap_freealldevs(pAllDevs);
}

void CCustomSnoop::Open(string strHost, string strUserName, string strPassword)
{
    int i;
    string strSource;
    int nAdapterIndex;
    pcap_rmtauth Auth;
    pcap_if_t *pDev;
    pcap_if_t *pAllDevs;
    char szErrBuf[PCAP_ERRBUF_SIZE];
    CVDStringList strAdapterNames;
    CVDStringList strAdapterDescriptions;

    if (m_bActive)
    {
        m_strError = "Snoop Error: Already open";
        return;
    }
    strSource = "rpcap://" + strHost + "/";
    Auth.type = RPCAP_RMTAUTH_PWD;
    Auth.username = (char*)strUserName;
    Auth.password = (char*)strPassword;
    i = pcap_findalldevs_ex((char*)strSource, &Auth, &pAllDevs, szErrBuf);
    if (i != 0) // if (error occured
    {
        m_strError = szErrBuf;
        return;
    }
    pDev = pAllDevs;
    while (pDev != NULL)
    {
        strAdapterNames.Add((string)pDev->name);
        strAdapterDescriptions.Add((string)pDev->description);
        pDev = pDev->next;
    }
    nAdapterIndex = -1;
    OnGetRemoteAdapterInfo(this, &strAdapterNames, &strAdapterDescriptions, nAdapterIndex);
    if (nAdapterIndex != -1)
    pDev = pAllDevs;
    for (i = 0 ; i < nAdapterIndex; i++)
        pDev = pDev->next;
    if (pDev == NULL)
        m_strError = "Snoop Error: Cancel By User or Invalid AdapterName";
    else
    {
        strSource = pDev->name;
        DoOpen((char*)strSource, &Auth, pDev);
        if (m_bActive && m_bCaptureEnabled)
            CreateSnoopThread();
    }
    pcap_freealldevs(pAllDevs);
}

bool CCustomSnoop::LoadFromFile(string strFileName)
{
    int i;
    PCAP_PKTHDR *pPKTHdr;
    ETH_HDR *pPKTData;

    if (m_bActive)
    {
        m_strError = "Snoop Error: Already open";
        return false;
    }
    if (strFileName == (string)"")
    {
        m_strError = "Snoop Error: Invalid FileName";
        return false;
    }
    DoOpen((char*)("file://" + strFileName), NULL, NULL);
    if (!m_bActive) return false;;
    if (m_bCaptureEnabled)
    {
        while (true)
        {
            i = pcap_next_ex(m_pPcap, &pPKTHdr, (const u_char**)&pPKTData);
            switch (i) {
                case -2: break; // EOF was reached reading from an offline capture
                case -1: // if an error occurred
                    m_strError = "Snoop Error: pcap_next_ex return -1";
                    break;
                case 0 : continue; // if (the timeout set with pcap_open_live() has elapsed.
            }
            m_pPKTHdr = pPKTHdr;
            m_pPKTData = pPKTData;
            OnCaptureCB();
        }
    }
    m_bActive = false;
    pcap_close(m_pPcap);
    return true;
}

void CCustomSnoop::Close()
{
	if (!m_bActive)
		return;
	if (m_pThread != NULL)
	{
		m_pThread->Terminate();
		m_pThread->Close();
		delete m_pThread;
		m_pThread = NULL;
	}
	pcap_close(m_pPcap);
	m_bActive = false;
}

// ----------------------------------------------------------------------------
// CSnoop
// ----------------------------------------------------------------------------
void CSnoop::OnCaptureCB()
{
	OnCapture(this, m_pPKTHdr, m_pPKTData);
}

void CSnoop::OnCapture(
        void        *pSender,
        PCAP_PKTHDR *pPKTHdr,
        ETH_HDR     *pETHHdr)
{
	// you must implement this method
	GTRACE("[Snoop.cpp] ***************************************");
	GTRACE("[Snoop.cpp] CSnoop::OnCapture virtual function call");
	GTRACE("[Snoop.cpp] ***************************************");
}

// ----------------------------------------------------------------------------
// CSnoopIP
// ----------------------------------------------------------------------------
void CSnoopIP::OnCaptureCB()
{
	IP_HDR *pIPHdr;

	if (!snoopIsIP(m_pPKTData, &pIPHdr)) return;
	OnCaptureIP(this, m_pPKTHdr, m_pPKTData, pIPHdr);
}

void CSnoopIP::OnCaptureIP(
	void        *pSender,
	PCAP_PKTHDR *pPKTHdr,
	ETH_HDR     *pETHHdr,
	IP_HDR      *pIPHdr)
{
	// you must implement this method
	GTRACE("[Snoop.cpp] *******************************************");
	GTRACE("[Snoop.cpp] CSnoopIP::OnCaptureIP virtual function call");
	GTRACE("[Snoop.cpp] *******************************************");
}

// ----------------------------------------------------------------------------
// CSnoopARP
// ----------------------------------------------------------------------------
void CSnoopARP::OnCaptureCB()
{
	ARP_HDR *pARPHdr;

	if (!snoopIsARP(m_pPKTData, &pARPHdr)) return;
	OnCaptureARP(this, m_pPKTHdr, m_pPKTData, pARPHdr);
}

void CSnoopARP::OnCaptureARP(
	void        *pSender,
	PCAP_PKTHDR *pPKTHdr,
	ETH_HDR     *pETHHdr,
	ARP_HDR     *pARPHdr)
{
	// you must implement this method
	GTRACE("[Snoop.cpp] *********************************************");
	GTRACE("[Snoop.cpp] CSnoopARP::OnCaptureARP virtual function call");
	GTRACE("[Snoop.cpp] *********************************************");
}

// ----------------------------------------------------------------------------
// CSnoopICMP
// ----------------------------------------------------------------------------
void CSnoopICMP::OnCaptureCB()
{
	IP_HDR   *pIPHdr;
	ICMP_HDR *pICMPHdr;

	if (!snoopIsIP(m_pPKTData, &pIPHdr)) return;
	if (!snoopIsICMP(pIPHdr, &pICMPHdr)) return;
	OnCaptureICMP(this, m_pPKTHdr, m_pPKTData, pIPHdr, pICMPHdr);
}

void CSnoopICMP::OnCaptureICMP(
	void        *pSender,
	PCAP_PKTHDR *pPKTHdr,
	ETH_HDR     *pETHHdr,
	IP_HDR      *pIPHdr,
	ICMP_HDR    *pICMPHdr)
{
	// you must implement this method
	GTRACE("[Snoop.cpp] ***********************************************");
	GTRACE("[Snoop.cpp] CSnoopICMP::OnCaptureICMP virtual function call");
	GTRACE("[Snoop.cpp] ***********************************************");
}

// ----------------------------------------------------------------------------
// CSnoopTCP
// ----------------------------------------------------------------------------
void CSnoopTCP::OnCaptureCB()
{
	IP_HDR  *pIPHdr;
	TCP_HDR *pTCPHdr;

	if (!snoopIsIP(m_pPKTData, &pIPHdr)) return;
	if (!snoopIsTCP(pIPHdr, &pTCPHdr)) return;
	OnCaptureTCP(this, m_pPKTHdr, m_pPKTData, pIPHdr, pTCPHdr);
}

void CSnoopTCP::OnCaptureTCP(
	void        *pSender,
	PCAP_PKTHDR *pPKTHdr,
	ETH_HDR     *pETHHdr,
	IP_HDR      *pIPHdr,
	TCP_HDR     *pTCPHdr)
{
	// you must implement this method
	GTRACE("[Snoop.cpp] *********************************************");
	GTRACE("[Snoop.cpp] CSnoopTCP::OnCaptureTCP virtual function call");
	GTRACE("[Snoop.cpp] *********************************************");
}

// ----------------------------------------------------------------------------
// CSnoopTCPData
// ----------------------------------------------------------------------------
void CSnoopTCPData::OnCaptureCB()
{
	IP_HDR  *pIPHdr;
	TCP_HDR *pTCPHdr;
	char    *pTCPData;
	int      nTCPDataLen;

	if (!snoopIsIP(m_pPKTData, &pIPHdr)) return;
	if (!snoopIsTCP(pIPHdr, &pTCPHdr)) return;
	if (!snoopIsTCPData(pIPHdr, pTCPHdr, &pTCPData, &nTCPDataLen)) return;
	OnCaptureTCPData(this, m_pPKTHdr, m_pPKTData, pIPHdr, pTCPHdr, pTCPData, nTCPDataLen);
}

void CSnoopTCPData::OnCaptureTCPData(
	void        *pSender,
	PCAP_PKTHDR *pPKTHdr,
	ETH_HDR     *pETHHdr,
	IP_HDR      *pIPHdr,
	TCP_HDR     *pTCPHdr,
	char        *pTCPData,
	int          nTCPDataLen)
{
	// you must implement this method
	GTRACE("[Snoop.cpp] *****************************************************");
	GTRACE("[Snoop.cpp] CSnoopTCPData::OnCaptureTCPData virtual function call");
	GTRACE("[Snoop.cpp] *****************************************************");
}

// ----------------------------------------------------------------------------
// CSnoopUDP
// ----------------------------------------------------------------------------
void CSnoopUDP::OnCaptureCB()
{
	IP_HDR  *pIPHdr;
	UDP_HDR *pUDPHdr;

	if (!snoopIsIP(m_pPKTData, &pIPHdr)) return;
	if (!snoopIsUDP(pIPHdr, &pUDPHdr)) return;
	OnCaptureUDP(this, m_pPKTHdr, m_pPKTData, pIPHdr, pUDPHdr);
}

void CSnoopUDP::OnCaptureUDP(
	void        *pSender,
	PCAP_PKTHDR *pPKTHdr,
	ETH_HDR     *pETHHdr,
	IP_HDR      *pIPHdr,
	UDP_HDR     *pUDPHdr)
{
	// you must implement this method
	GTRACE("[Snoop.cpp] *********************************************");
	GTRACE("[Snoop.cpp] CSnoopUDP::OnCaptureUDP virtual function call");
	GTRACE("[Snoop.cpp] *********************************************");
}

// ----------------------------------------------------------------------------
// CSnoopUDPData
// ----------------------------------------------------------------------------
void CSnoopUDPData::OnCaptureCB()
{
	IP_HDR  *pIPHdr;
	UDP_HDR *pUDPHdr;
	char    *pUDPData;
	int      nUDPDataLen;

	if (!snoopIsIP(m_pPKTData, &pIPHdr)) return;
	if (!snoopIsUDP(pIPHdr, &pUDPHdr)) return;
	if (!snoopIsUDPData(pIPHdr, pUDPHdr, &pUDPData, &nUDPDataLen)) return;
	OnCaptureUDPData(this, m_pPKTHdr, m_pPKTData, pIPHdr, pUDPHdr, pUDPData, nUDPDataLen);
}

void CSnoopUDPData::OnCaptureUDPData(
	void        *pSender,
	PCAP_PKTHDR *pPKTHdr,
	ETH_HDR     *pETHHdr,
	IP_HDR      *pIPHdr,
	UDP_HDR     *pUDPHdr,
	char        *pUDPData,
	int          nUDPDataLen)
{
	// you must implement this method
	GTRACE("[Snoop.cpp] *****************************************************");
	GTRACE("[Snoop.cpp] CSnoopTCPData::OnCaptureUDPData virtual function call");
	GTRACE("[Snoop.cpp] *****************************************************");
}

// ----------------------------------------------------------------------------
// CSnoopStatistics
// ----------------------------------------------------------------------------
void CSnoopStatistics::OnCaptureCB()
{
	__int64 nPackets;
	__int64 nBytes;
	__int64 *p;

	p = (__int64*)m_pPKTData;
	nPackets = *p++;
	nBytes = *p;
	OnCaptureStatistics(this, m_pPKTHdr, nPackets, nBytes);
}

CSnoopStatistics::CSnoopStatistics()
{
}

CSnoopStatistics::~CSnoopStatistics()
{
	CCustomSnoop::Open();
	if (m_bActive)
		pcap_setmode(m_pPcap, MODE_STAT);
}

void Open();

void CSnoopStatistics::OnCaptureStatistics(
		void         *pSender,
		PCAP_PKTHDR  *pPKTHdr,
		__int64       nPackets,
		__int64       nBytes)
{
}

// ----------------------------------------------------------------------------
// CSnoopDump
// ----------------------------------------------------------------------------
// gilgil temp 2006.10.24

// ----------------------------------------------------------------------------
// CSnoopMyNetwork
// ----------------------------------------------------------------------------
CSnoopMyNetwork::CSnoopMyNetwork()
{
	m_pSnoop = new CSnoop;
	Refresh();
}

CSnoopMyNetwork::~CSnoopMyNetwork()
{
	if (m_pSnoop != NULL)
	{
		delete m_pSnoop;
		m_pSnoop = NULL;
	}
}

void CSnoopMyNetwork::Refresh()
{
	IP_ADAPTER_INFO *p, *pAdapterInfo;
	ULONG uOutBufLen;
	DWORD dwRes;

	if (m_pSnoop == NULL)
	{
		return;
	}
	m_uIP = 0;
	m_sMac = snoopCleanMac();
	m_uSubnetMask = 0;
	m_uGateway = 0;
	if (m_pSnoop->m_strAdapterName == (string)"")
	{
		return;
	}
	pAdapterInfo = NULL;
	uOutBufLen = 0;
	dwRes = GetAdaptersInfo(pAdapterInfo, &uOutBufLen);
	if (dwRes == ERROR_BUFFER_OVERFLOW)
	{
		pAdapterInfo = (IP_ADAPTER_INFO*)malloc(uOutBufLen);
		dwRes = GetAdaptersInfo(pAdapterInfo, &uOutBufLen);
	}
	if (dwRes != ERROR_SUCCESS)
	{
		return;
	}
	p = pAdapterInfo;
	while (p != NULL)
	{
		if (strstr((char*)m_pSnoop->m_strAdapterName, (char*)p->AdapterName) != NULL)
			break;
		p = p->Next;
	}
	if (p != NULL)
	{
		if (p->IpAddressList.IpAddress.String[0] != '\0')
			m_uIP = snoopStr2IP(p->IpAddressList.IpAddress.String);
		memcpy(m_sMac.s, p->Address, SNOOP_MAC_SIZE);
		if (p->IpAddressList.IpMask.String[0] != '\0')
			m_uSubnetMask = snoopStr2IP(p->IpAddressList.IpMask.String);
		if (p->GatewayList.IpAddress.String[0] != '\0')
			m_uGateway = snoopStr2IP(p->GatewayList.IpAddress.String);
	}
	if (pAdapterInfo != NULL) free(pAdapterInfo);
}

int CSnoopMyNetwork::GetAdapterIndex()
{
	if (m_pSnoop == NULL)
	{
		return -1;
	}
	return m_pSnoop->m_nAdapterIndex;
}

void   CSnoopMyNetwork::SetAdapterIndex(int nValue)
{
	int nOldAdapterIndex;

	if (m_pSnoop == NULL)
	{
		return;
	}
	nOldAdapterIndex = m_pSnoop->m_nAdapterIndex;
	m_pSnoop->SetAdapterIndex(nValue);
	if (m_pSnoop->m_nAdapterIndex == nOldAdapterIndex) return;
	Refresh();
}

string CSnoopMyNetwork::GetAdapterName()
{
	if (m_pSnoop == NULL)
	{
		return ""; 
	}
	return m_pSnoop->m_strAdapterName;
}

string CSnoopMyNetwork::GetAdapterDescription()
{
	if (m_pSnoop == NULL)
	{
		return ""; 
	}
	return m_pSnoop->m_strAdapterDescription;
}

void   CSnoopMyNetwork::SetAdapterName(string strValue)
{
	if (m_pSnoop == NULL)
	{
		return;
	}
	m_pSnoop->SetAdapterName(strValue);
	Refresh();
}

void   CSnoopMyNetwork::SetAdapterDescription(string strValue)
{
	if (m_pSnoop == NULL)
	{
		return;
	}
	m_pSnoop->SetAdapterDescription(strValue);
	Refresh();
}

CVDStringList* CSnoopMyNetwork::GetAdapterNames()
{
	if (m_pSnoop == NULL)
	{
		return NULL;
	}
	return &m_pSnoop->m_strAdapterNames;
}

CVDStringList* CSnoopMyNetwork::GetAdapterDescriptions()
{
	if (m_pSnoop == NULL)
	{
		return NULL;
	}
	return &m_pSnoop->m_strAdapterDescriptions;
}

string CSnoopMyNetwork::GetStrIP()
{
	return snoopIP2Str(m_uIP);
}

string CSnoopMyNetwork::GetStrMac()
{
	return snoopMac2Str(m_sMac);
}

string CSnoopMyNetwork::GetStrGateway()
{
	return snoopIP2Str(m_uGateway);
}

string CSnoopMyNetwork::GetStrSubnetMask()
{
	return snoopIP2Str(m_uSubnetMask);
}

void   CSnoopMyNetwork::SetStrIP(string strValue)
{
	// do nothing
}

void   CSnoopMyNetwork::SetStrMac(string strValue)
{
	// do nothing
}

void   CSnoopMyNetwork::SetStrGateway(string strValue)
{
	// do nothing
}

void   CSnoopMyNetwork::SetStrSubnetMask(string strValue)
{
	// do nothing
}

// ----------------------------------------------------------------------------
// General Functions
// ----------------------------------------------------------------------------
bool snoopIsIP(ETH_HDR *pETHHdr, IP_HDR **ppIPHdr)
{
	if (ntohs(pETHHdr->Protocol) != PROTO_IP)
		return false;
	if (ppIPHdr != NULL)
		*ppIPHdr = (IP_HDR*)(UINT32(pETHHdr) + sizeof(ETH_HDR));
	return true;
}

bool snoopIsARP(ETH_HDR *pETHHdr, ARP_HDR **ppARPHdr)
{
	if (ntohs(pETHHdr->Protocol) != PROTO_ARP)
		return false;
	if (ppARPHdr != NULL)
		*ppARPHdr = (ARP_HDR*)(UINT32(pETHHdr) + sizeof(ETH_HDR));
	return true;
}

bool snoopIsICMP(IP_HDR *pIPHdr, ICMP_HDR **ppICMPHdr)
{
	if (pIPHdr->Protocol != PROTO_ICMP)
		return false;
	if (ppICMPHdr != NULL)
		*ppICMPHdr = (ICMP_HDR*)(UINT32(pIPHdr) + sizeof(IP_HDR));
	return true;
}

bool snoopIsTCP(IP_HDR *pIPHdr, TCP_HDR **ppTCPHdr)
{
	if (pIPHdr->Protocol != PROTO_TCP)
		return false;
	if (ppTCPHdr != NULL)
		*ppTCPHdr = (TCP_HDR*)(UINT32(pIPHdr) + sizeof(IP_HDR));
	return true;
}

bool snoopIsUDP(IP_HDR *pIPHdr, UDP_HDR **ppUDPHdr)
{
	if (pIPHdr->Protocol != PROTO_UDP)
		return false;
	if (ppUDPHdr != NULL)
		*ppUDPHdr = (UDP_HDR*)(UINT32(pIPHdr) + sizeof(IP_HDR));
	return true;
}

bool snoopIsTCPData(IP_HDR *pIPHdr, TCP_HDR *pTCPHdr, char **ppTCPData, int *pTCPDataLen)
{
	int  nTCPHdrLen;
	char *pData;
	int  nLength;

	nTCPHdrLen = ((pTCPHdr->Off_Rsvd & 0xF0) >> 4) * sizeof(UINT32);
	pData = (char*)(UINT32(pTCPHdr) + UINT32(nTCPHdrLen));
	nLength = ntohs(pIPHdr->Len) - sizeof(TCP_HDR) - nTCPHdrLen;
	if (ppTCPData != NULL) *ppTCPData = pData;
	if (pTCPDataLen != NULL) *pTCPDataLen = nLength;
	return nLength > 0;
}

bool snoopIsUDPData(IP_HDR *pIPHdr, UDP_HDR *pUDPHdr, char **ppUDPData, int *pUDPDataLen)
{
	int nUDPDataLen;

	if (ppUDPData != NULL) *ppUDPData = (char*)(UINT32(pUDPHdr) + sizeof(UDP_HDR));
	nUDPDataLen = ntohs(pUDPHdr->Len) - sizeof(UDP_HDR);
	if (pUDPDataLen != NULL) *pUDPDataLen = nUDPDataLen;
	return nUDPDataLen > 0;
}

string snoopMac2Str(MACADDR sMac)
{
	UCHAR ch1, ch2;
	int i, nIndex;
	char szBuffer[SNOOP_MAC_SIZE * 3]; // enough size
	
	nIndex = 0;
	for (i = 0; i < SNOOP_MAC_SIZE; i++)
	{
		ch1 = sMac.s[i] & 0xF0;
		ch1 = ch1 >> 4;
		if (ch1 > 9)
			ch1 = ch1 + 'A' - 10;
		else
			ch1 = ch1 + '0';
		ch2 = sMac.s[i] & 0x0F;
		if (ch2 > 9)
			ch2 = ch2 + 'A' - 10;
		else
			ch2 = ch2 + '0';
		szBuffer[nIndex++] = ch1;
		szBuffer[nIndex++] = ch2;
		if (i == 2)
			szBuffer[nIndex++] = '-';
	}
	szBuffer[nIndex] = '\0';
	return (string(szBuffer));
}

string  snoopMac2StrFormat(MACADDR sMac, string strFormat)
{
	char szBuffer[256]; // enough size
	sprintf(szBuffer, (char*)strFormat, sMac.s);
	return (string(szBuffer));
}

string snoopIP2Str(UINT32 IP)
{
	IP = htonl(IP);
	string s = (char*)inet_ntoa(*((in_addr*)&IP));
	return s;
	// ------ gilgil temp 2006.10.04 -----
	/*
	char szBuffer[16];
	sprintf(szBuffer, "%d.%d.%d.%d",
	    (IP & 0xFF000000) >> 24,
	    (IP & 0x00FF0000) >> 16,
	    (IP & 0x0000FF00) >> 8,
	    (IP & 0x000000FF) >> 0);
	return (string(szBuffer));
	*/
	// -----------------------------------
}

MACADDR snoopStr2Mac(string s)
{
	int i;
	UCHAR *p;
	UCHAR ch1, ch2;
	MACADDR sMac;

	p = (UCHAR*)(char*)s;
	for (i = 0 ; i < SNOOP_MAC_SIZE; i++)
	{
		ch1 = *p++;
		if (ch1 >= 'a') ch1 = ch1 - 'a' + 'A';
		if (ch1 >= 'A')
			ch1 = ch1 - 'A' + 10;
		else
			ch1 = ch1 - '0';
		ch2 = *p++;
		if (ch2 >= 'a') ch2 = ch2 - 'a' + 'A';
		if (ch2 >= 'A')
			ch2 = ch2 - 'A' + 10;
		else
			ch2 = ch2 - '0';
		sMac.s[i] = (ch1 << 4) + ch2;
		while (*p == '-' || *p == ':') p++;
	}
	return sMac;
}

UINT32 snoopStr2IP(string s)
{
	return ntohl(inet_addr((char*)s));
}

MACADDR snoopCleanMac()
{
	int i;
	MACADDR sMac;

	for (i = 0; i < SNOOP_MAC_SIZE; i++)
		sMac.s[i] = '\0';
	return sMac;
}

bool snoopIsCleanMac(MACADDR *pMac)
{
	int i;

	for (i = 0; i < SNOOP_MAC_SIZE; i++)
	{
		if (pMac->s[i] != '\0')
		{
			return false;
		}
	}
	return true;
}

int snoopCompareMac(UCHAR *pMac1, UCHAR *pMac2)
{
	int i;

	for (i = 0; i < SNOOP_MAC_SIZE; i++)
	{
		if (pMac1[i] == pMac2[i]) continue;
		if (pMac1[i] < pMac2[i])
		{
			return -1;
		} else
		{
			return 1;
		}
	}
	return 0;
}

//
// All IPHeader field except IPHeader.Checksum
//
UINT16 snoopIPChecksum(IP_HDR *pIPHdr)
{
	int i;
	UINT32 Sum;
	UINT16 *p;

	Sum = 0;

	// Add IPHdr Buffer as array of UINT16
	p = (UINT16*)(pIPHdr);
	for(i=0; i < sizeof(IP_HDR) / 2; i++)
	{
		Sum += ntohs(*p);
		p++;
	}

	// Decrease Checksum from Sum
	Sum -= ntohs(pIPHdr->Checksum);

	// Recalculate Sum
	while(Sum >> 16)
	{
		Sum = (Sum & 0xFFFF) + (Sum >> 16);
	}
	Sum = ~Sum;

	return (UINT16)Sum;
}

//
// All TCPHdr field except TCPHdr.Checksum
// IPHdr.Source, IPHdr.Destination, TCPHdrDataLeng
//
UINT16 snoopTCPChecksum(IP_HDR *pIPHdr, TCP_HDR *pTCPHdr)
{
	int i;
	int nTCPHdrDataLen;
	UINT32 Source, Destination;
	UINT32 Sum;
	UINT16 *p;

	nTCPHdrDataLen = ntohs(pIPHdr->Len) - sizeof(IP_HDR);
	Sum = 0;

	// Add TCPHdr and Data Buffer as array of UIN16
	p = (UINT16*)(pTCPHdr);
	for (i = 0; i < nTCPHdrDataLen / 2; i++)
	{
		Sum = Sum + htons(*p);
		p++;
	}

	// if Length is OddNumber, Add Last Data
	if ((nTCPHdrDataLen / 2) * 2 != nTCPHdrDataLen)
		Sum += (htons(*p) & 0xFF00);

	// Decrease Checksum from Sum
	Sum -= ntohs(pIPHdr->Checksum);

	// Add Source Address
	Source = ntohl(pIPHdr->Source);
	Sum = Sum + ((Source & 0xFFFF0000) >> 16) + (Source & 0x0000FFFF);

	// Add Destination Addres
	Destination = ntohl(pIPHdr->Destination);
	Sum = Sum + ((Destination & 0xFFFF0000) >> 16) + (Destination & 0x0000FFFF);

	// Add Extra Information
	Sum = Sum + PROTO_TCP + (UINT32)(nTCPHdrDataLen);

	// Recalculate Sum
	while(Sum >> 16)
	{
		Sum = (Sum & 0xFFFF) + (Sum >> 16);
	}
	Sum = ~Sum;

	return (UINT16)Sum;
}

//
// All UDPHeader field except UDPHdr.Checksum
// IPHdr.Source, IPHdr.Destination, UDPHdrDataLen
//
UINT16 snoopUDPChecksum(IP_HDR *pIPHdr, UDP_HDR *pUDPHdr)
{
	int i;
	int nUDPHdrDataLen;
	UINT16 *p;
	UINT32 Source, Destination;
	UINT32 Sum;

	nUDPHdrDataLen = ntohs(pUDPHdr->Len);
	Sum = 0;

	// Add pUDPHdr & Data Buffer as array of UINT16
	p = (UINT16*)pUDPHdr;
	for (i = 0; i < nUDPHdrDataLen / 2; i++)
	{
		Sum += htons(*p);
		p++;
	}

	// if (Length is OddNumber, Add Last Data
	if ((nUDPHdrDataLen / 2) * 2 != nUDPHdrDataLen)
		Sum += (htons(*p) & 0xFF00);

	// Decrease Checksum from Sum
	Sum -= ntohs(pUDPHdr->Checksum);

	// Add uSource Address
	Source = ntohl(pIPHdr->Source);
	Sum += ((Source & 0xFFFF0000) >> 16) + (Source & 0x0000FFFF);

	// Add uDestination Addres
	Destination = ntohl(pIPHdr->Destination);
	Sum += ((Destination & 0xFFFF0000) >> 16) + (Destination & 0x0000FFFF);

	// Add Extra Information
	Sum += PROTO_UDP + (UINT32)nUDPHdrDataLen;

	// Recalculate uSum
	while ((Sum >> 16) > 0)
		Sum = (Sum & 0xFFFF) + (Sum >> 16);
	Sum = ~Sum;

	return (UINT16)Sum;
}

//
// Calculate Checksum on condition that only one value(UINT16) is changed.
//
UINT16 snoopRecalculateChecksum(UINT16 uOldChecksum, UINT16 uOldValue, UINT16 uNewValue)
{
	UINT32 uSum;

	uSum = uOldValue + (~uNewValue & 0xFFFF);
	uSum += uOldChecksum;
	uSum = (uSum & 0xffff) + (uSum >> 16);
	return (UINT16)(uSum + (uSum >> 16));
}

int snoopSendPacket(pcap *pcap, char *pBuffer, int nSize)
{
	return pcap_sendpacket(pcap, (const u_char*)pBuffer, nSize);
}
