#include "SnoopSpecial.h"

// ----------------------------------------------------------------------------
// CSnoopFindMac
// ----------------------------------------------------------------------------
void CSnoopFindMac::OnCaptureCB()
{
	ARP_HDR *pARPHdr;

	if (!snoopIsARP(m_pPKTData, &pARPHdr)) return;
	if ((ntohs(pARPHdr->Operation) == ARP_REPLY) &&
		(snoopCompareMac(&m_pPKTData->Destination, &m_pSnoopMyNetwork->m_sMac) == 0) &&
		(UINT32(ntohl(pARPHdr->SenderIP)) == m_pSnoopMyNetwork->m_uIP))
	{
		OnCaptureFindMac(this, ntohl(pARPHdr->SenderIP), &pARPHdr->SenderHA);
	}
}

CSnoopFindMac::CSnoopFindMac()
{
	m_pSnoopMyNetwork = new CSnoopMyNetwork;
}

CSnoopFindMac::~CSnoopFindMac()
{
	if (m_pSnoopMyNetwork != NULL)
	{
		delete m_pSnoopMyNetwork;
		m_pSnoopMyNetwork = NULL;
	}
}

void CSnoopFindMac::Open()
{
	// Check if sniffing is active
	if (m_bActive) return;
	m_pSnoopMyNetwork->SetAdapterIndex(m_nAdapterIndex);
	m_strFilter = "arp host " + m_pSnoopMyNetwork->GetStrIP();
	CCustomSnoop::Open();
}

void CSnoopFindMac::SendRequest(UINT32 uIP)
{
	ETH_HDR sETHHdr;
	ARP_HDR      sARPHeader;

	if (! m_bActive) return;
	// Set Ethernet Header
	sETHHdr.Destination = snoopStr2Mac("FFFFFF-FFFFFFF");
	sETHHdr.Source = m_pSnoopMyNetwork->m_sMac;
	sETHHdr.Protocol = htons(PROTO_ARP);

	// Set ARP Header
	sARPHeader.HardwareType = htons(1);
	sARPHeader.ProtocolType = htons(2048);
	sARPHeader.HLen = 6;
	sARPHeader.PLen = 4;
	sARPHeader.Operation = htons(ARP_REQUEST);
	sARPHeader.SenderHA = m_pSnoopMyNetwork->m_sMac;
	sARPHeader.SenderIP = htonl(m_pSnoopMyNetwork->m_uIP);
	sARPHeader.TargetHA = snoopCleanMac();
	sARPHeader.TargetIP = htonl(uIP);

	// Send Raw Packet
	snoopSendARP(m_pPcap, &sETHHdr, &sARPHeader);
}

void CSnoopFindMac::OnCaptureFindMac(
		void *pSender, UINT32 uIP, MACADDR *pMac)
{
	GTRACE("[SnoopSpecial.cpp] *****************************************************");
	GTRACE("[SnoopSpecial.cpp] CSnoopFindMac::OnCaptureFindMac virtual function call");
	GTRACE("[SnoopSpecial.cpp] *****************************************************");
}

// ----------------------------------------------------------------------------
// CARPTable
// ----------------------------------------------------------------------------
CARPTable::CARPTable()
{
	Clear();
}

CARPTable::~CARPTable()
{
}

void CARPTable::Clear()
{
	int i;

	for (i = 0; i < ARP_TABLE_COUNT; i++)
	{
		m_bActive[i] = false;
		m_uIP[i] = 0;
		m_sMac[i] = snoopCleanMac();
	}
}

int CARPTable::Add(UINT32 uIP, MACADDR *pMac)
{
	int i;

	if (Find(uIP) != -1) return -1;
	for (i = 0; i < ARP_TABLE_COUNT; i++)
	{
		if (! m_bActive[i])
		{
			m_bActive[i] = true;
			m_uIP[i] = uIP;
			m_sMac[i] = *pMac;
			return i;
		}
	}
	return -1;
}

int CARPTable::Find(UINT32 uIP)
{
	int i;

	for (i = 0; i < ARP_TABLE_COUNT; i++)
	{
		if (m_bActive[i] && m_uIP[i] == uIP)
		{
			return i;
		}
	}
	return -1;
}

// ----------------------------------------------------------------------------
// CARPSpoofTable
// ----------------------------------------------------------------------------
CARPSpoofTable::CARPSpoofTable()
{
	Clear();
}

CARPSpoofTable::~CARPSpoofTable()
{
}

void CARPSpoofTable::Clear()
{
	int i;

	for (i = 0; i < ARP_SPOOF_TABLE_COUNT; i++)
	{
		m_bActive[i] = false;
		m_uSenderIP[i] = 0;
		m_sSenderMac[i] = snoopCleanMac();
		m_uTargetIP[i] = 0;
		m_sTargetMac[i] = snoopCleanMac();
	}
}

int CARPSpoofTable::Add(UINT32 uSenderIP, UINT32 uTargetIP)
{
	int i;

	if (Find(uSenderIP, uTargetIP) != -1) return -1;
	for (i = 0;  i < ARP_SPOOF_TABLE_COUNT; i++)
		if (! m_bActive[i])
		{
			m_bActive[i] = true;
			m_uSenderIP[i] = uSenderIP;
			m_sSenderMac[i] = snoopCleanMac();
			m_uTargetIP[i] = uTargetIP;
			m_sTargetMac[i] = snoopCleanMac();
			return i;
		}
	GTRACE("[Snoop.cpp] can not add (%s, %s)",
		(char*)snoopIP2Str(uSenderIP), (char*)snoopIP2Str(uTargetIP));
	return -1;
}

int CARPSpoofTable::Update(UINT32 uSenderIP, MACADDR *pSenderMac, UINT32 uTargetIP, MACADDR *pTargetMac)
{
	int i;

	i = Find(uSenderIP, uTargetIP);
	if (i == -1) return -1;
	m_sSenderMac[i] = *pSenderMac;
	m_sTargetMac[i] = *pTargetMac;
	return i;
}

int CARPSpoofTable::Delete(int nIndex)
{
	if ((nIndex < 0) || (nIndex >= ARP_SPOOF_TABLE_COUNT)) return -1;
	m_bActive[nIndex] = false;
	m_uSenderIP[nIndex] = 0;
	m_sSenderMac[nIndex] = snoopCleanMac();
	m_uTargetIP[nIndex] = 0;
	m_sTargetMac[nIndex] = snoopCleanMac();
	return nIndex;
}

int CARPSpoofTable::Delete(UINT32 uSenderIP, UINT32 uTargetIP)
{
	return Delete(Find(uSenderIP, uTargetIP));
}

int CARPSpoofTable::Delete(UINT32 uSenderIP, MACADDR *pSenderMac, UINT32 uTargetIP, MACADDR *pTargetMac)
{
	return Delete(Find(uSenderIP, pSenderMac, uTargetIP, pTargetMac));
}

int CARPSpoofTable::Find(UINT32 uIP)
{
	return -1; // gilgil temp 2006.11.22
}

int CARPSpoofTable::Find(UINT32 uSenderIP, UINT32 uTargetIP)
{
	return -1; // gilgil temp 2006.11.22
}

int CARPSpoofTable::Find(UINT32 uSenderIP, MACADDR *pSenderMac, UINT32 uTargetIP, MACADDR *pTargetMac)
{
	return -1; // gilgil temp 2006.11.22
}

int CARPSpoofTable::Count()
{
	return -1; // gilgil temp 2006.11.22
}

